// ------------------------------------------------------------------------------------------------

/*
    This file is part of Style Sheet Chooser Plus.

    Style Sheet Chooser Plus is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Style Sheet Chooser Plus is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Style Sheet Chooser Plus; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// ------------------------------------------------------------------------------------------------
//
// Style Sheet Chooser Plus
//
// Copyright 2004-2006, OLab (http://www.olab.info/)
// Thanks to Rob Tougher (http://www.robtougher.com) for the original extension
// namely Stylesheet Chooser, on which this one is partly based
//
// This extension persists your choices for alternate stylesheets on
// a per-domain basis. For example, if you browse to http://domain.com/index1.html,
// choose the alternate stylesheet "Alt", and browse to http://domain.com/index2.html,
// the alternate stylesheet "Alt" will be used on the second page, index2.html.
// Furthermore, an icon in the status bar helps you to select it.
//
// Choices are persisted between window instances, so your choices will be remembered
// if you close Firefox and restart it.
//
// ------------------------------------------------------------------------------------------------

var SSCPLegacy = {
	/** Runs import process of old preferences
	  * (from Stylesheet Chooser, Stylesheet Chooser II or SSCP)
	  */
	run: function() {
		// Files
		this.rdfFilename = 'StyleSheetChooserPlus.rdf';
		this.sscFilename = 'StyleSheetChooser-0.1.txt';
		// Services
		this.prefsJsService = Components.classes['@mozilla.org/preferences-service;1'].getService(
					Components.interfaces.nsIPrefService).getBranch('stylesheetchooserplus.settings.');
		this.rdfService = Components.classes["@mozilla.org/rdf/rdf-service;1"].getService(
					Components.interfaces.nsIRDFService);
		this.directoryService = Components.classes["@mozilla.org/file/directory_service;1"].createInstance(
						Components.interfaces.nsIProperties);
		this.promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                        .getService(Components.interfaces.nsIPromptService);
        
		// Load rdfCacheFile
		var rdfLoaded = this.loadRdfCache();
		// Load SSC-style cache into the rdf tree
		var txtLoaded = this.loadOldCacheFile();
		// Save the rdf tree
		if (rdfLoaded) this.saveRdfCache();
		
		// Load preferences
		var prefsJsLoaded = this.loadPrefsJs();
		
		// Ask if user want to delete old preferences, only if something has been imported
		if (rdfLoaded || txtLoaded || prefsJsLoaded) {
			var button = this.promptService.confirmEx(null, SSCP.strings.GetStringFromName("confirmDelete.title"),
	                                SSCP.strings.GetStringFromName("confirmDelete.description"),
	                                this.promptService.STD_YES_NO_BUTTONS, null, null, null, null, {});

			if (button == 0) {
	    		// Delete SSC and RDF cache files
	    		if (rdfLoaded || txtLoaded) this.deleteOldCacheFiles();
	    		// Remove prefs.js properties
	    		if (prefsJsLoaded) this.removePrefsJs();
	    	}
	    }
		
		return true;
	},
	
	/** Imports sscp preferences from prefs.js (SSCP 2.0-) */
	loadPrefsJs: function() {
		var present = false;
		if (this.prefsJsService.prefHasUserValue('propagation')) {
			SSCP.prefService.setValue(SSCP.prefService.optionsTable, 'propagation', this.prefsJsService.getBoolPref('propagation'));
			present = true;
		}

		if (this.prefsJsService.prefHasUserValue('iconInDisabledMode')) {
			SSCP.prefService.setValue(SSCP.prefService.optionsTable, 'alwaysShowStatusButton', this.prefsJsService.getBoolPref('iconInDisabledMode'));
			present = true;
		}
		
		return present;
	},
	
	/** Removes sscp preferences from prefs.js (SSCP 2.0-) */
	removePrefsJs: function() {
		this.prefsJsService.deleteBranch("");
	},
	
	/** Loads in memory the rdf cache file (SSCP 0.2.3+) */
	loadRdfCache: function() {
		var present = false;
		// Load the file
		var filename = this.directoryService.get("ProfD", Components.interfaces.nsIFile);
		filename.append(this.rdfFilename);
	
		if (filename.exists()) {
			var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
			file.initWithPath(filename.path);
			
			this.rdfCacheFile = this.rdfService.GetDataSourceBlocking('file:///' + file.path);
			present = true;
		}
		
		return present;
	},
	
	/** Parses the in-memory rdf tree and add its values to the new backend
	  * This rdf tree may contain SSCP 0.2.3-, SSC and SSC II values,
	  * because we import txt preferences and rdf's.
	  */
	saveRdfCache: function() {
		// The rdf namespace
		var ns = "http://olab.free.fr/sscplus-syntax#";
		// The predicate
		var stylesheetProp = this.rdfService.GetResource(ns + "stylesheet");
		
		// Get all resources
		var allRes = this.rdfCacheFile.GetAllResources();
		while (allRes.hasMoreElements()) {
			var res = allRes.getNext();
			res.QueryInterface(Components.interfaces.nsIRDFResource);
			var target = this.rdfCacheFile.GetTarget(res, stylesheetProp, true);
			target.QueryInterface(Components.interfaces.nsIRDFLiteral);
			
			// Get website (the resource name without the namespace)
			var site = (res.Value).replace(ns, '');
			var stylesheet = target.Value;
			
			// Add the value to backend, only if this is not the default value and the site really exists
			if (stylesheet != '_default' && site != '')
				SSCP.prefService.setValue(SSCP.prefService.authorStylesTable, site, stylesheet);
		}
	},
	
	/** Imports txt-type preferences (SSCP 0.2.3- and SSC and SSC II) into an rdf tree */
	loadOldCacheFile: function() {
		/** See if import is needed */
		var needImport = false;

		var profD = this.directoryService.get("ProfD", Components.interfaces.nsIFile);
		var sscFile = profD.clone();

		sscFile.append(this.sscFilename);

		// First launch of sscp, with previous ssc installation
		if (sscFile.exists()) {
			needImport = true;
		}

		if (needImport) {
			var selectionCache = new Array();

			var fileInputStream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(
											Components.interfaces.nsIFileInputStream);

			fileInputStream.init(sscFile, 0x01, 00004, null);

			var scriptableInputStream = Components.classes["@mozilla.org/scriptableinputstream;1"].createInstance(
										Components.interfaces.nsIScriptableInputStream);

			scriptableInputStream.init(fileInputStream);

			// Parse the file
			var sData = scriptableInputStream.read(scriptableInputStream.available());
			var items = sData.split('\n');
			for (var i = 0; i < items.length - 1; i+=2) {
				var key = items[i];
				var value = items[i+1];
				selectionCache[key] = value;
			}

			fileInputStream.close();

			// Add the old cache to the new backend
			for (key in selectionCache) {
				SSCP.prefService.setValue(SSCP.prefService.authorStylesTable, key, selectionCache[key]);
			}
		}
		
		return needImport;
	},
	
	/** Deletes old preferences files from profile */
	deleteOldCacheFiles: function() {
		var profD = this.directoryService.get("ProfD", Components.interfaces.nsIFile);
		var sscFile = profD.clone();
		var sscpFile = profD.clone();

		sscFile.append(this.sscFilename);
		sscpFile.append(this.rdfFilename);
		
		// Delete SSC-style cache file
		if (sscFile.exists()) {
			sscFile.remove(false);
		}
		// Delete rdf cache file
		if (sscpFile.exists()) {
			sscpFile.remove(false);
		}
	}
};
