// ------------------------------------------------------------------------------------------------

/*
    This file is part of Style Sheet Chooser Plus.

    Style Sheet Chooser Plus is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Style Sheet Chooser Plus is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Style Sheet Chooser Plus; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// ------------------------------------------------------------------------------------------------
//
// Style Sheet Chooser Plus
//
// Copyright 2004-2006, OLab (http://www.olab.info/)
// Thanks to Rob Tougher (http://www.robtougher.com) for the original extension
// namely Stylesheet Chooser, on which this one is partly based
//
// This extension persists your choices for alternate stylesheets on
// a per-domain basis. For example, if you browse to http://domain.com/index1.html,
// choose the alternate stylesheet "Alt", and browse to http://domain.com/index2.html,
// the alternate stylesheet "Alt" will be used on the second page, index2.html.
// Furthermore, an icon in the status bar helps you to select it.
//
// Choices are persisted between window instances, so your choices will be remembered
// if you close Firefox and restart it.
//
// ------------------------------------------------------------------------------------------------

var SSCPOptions = {
	/* Initializes preferences dialog */
	init: function() {
		this.propagationElt = document.getElementById('propagation');
		this.keepDisabledStateElt = document.getElementById('keepDisabledState');
		this.alwaysShowStatusButtonElt = document.getElementById('alwaysShowStatusButton');
		
		// Loads preferences
		this.propagationElt.checked = SSCP.prefService.getValue(SSCP.prefService.optionsTable, 'propagation', true); // default value : true
		this.keepDisabledStateElt.checked = SSCP.prefService.getValue(SSCP.prefService.optionsTable, 'keepDisabledState', false); // default value : false
		this.alwaysShowStatusButtonElt.checked = SSCP.prefService.getValue(SSCP.prefService.optionsTable, 'alwaysShowStatusButton', true); // default value : true
		
		// Fill the stylesheets lists
		var styleTypes = [SSCP.prefService.authorStylesTable];
		for (var i=0; i<styleTypes.length; i++) {
			var styles = SSCP.prefService.getAllValues(styleTypes[i]);
			var listbox = document.getElementById(styleTypes[i]);
			if (listbox) {
				for (var j=0; j<styles.length; j++) {
					var listitem = document.createElement('listitem');
					var listcellSite = document.createElement('listcell');
					var listcellStylesheet = document.createElement('listcell');
					
					listcellSite.setAttribute("label", styles[j][0]);
					var stylesheet = styles[j][1];

					if (stylesheet == '_nostyle') {
						// Get label from popup menu
						var noStyleMenuElt = window.opener.document.getElementById('nostyle');
						if (noStyleMenuElt) {
							stylesheet = noStyleMenuElt.getAttribute('label');
						}
					}

					listcellStylesheet.setAttribute("label", stylesheet);
					
					listitem.appendChild(listcellSite);
					listitem.appendChild(listcellStylesheet);
					
					listbox.appendChild(listitem);
				}
			}
		}
	},

	/* Saves user preferences */
	save: function() {
		SSCP.prefService.setValue(SSCP.prefService.optionsTable, 'propagation', this.propagationElt.checked);
		SSCP.prefService.setValue(SSCP.prefService.optionsTable, 'keepDisabledState', this.keepDisabledStateElt.checked);
		SSCP.prefService.setValue(SSCP.prefService.optionsTable, 'alwaysShowStatusButton', this.alwaysShowStatusButtonElt.checked);
	},
	
	/* Deletes a style from the cache
	 * @param id The styles listbox id from which the user asked a deletion
	 */
	delStyle: function(id) {
	   var list = document.getElementById(id);
	   if (list) {
    		var count = list.selectedCount;
    		while (count--) {
    			var style = list.selectedItems[0];
    			for (var j=0; j<style.childNodes.length; j++) {
    				if (style.childNodes[j].tagName == 'listcell') {
    					var key = style.childNodes[j].getAttribute('label');
    					if (key) {
    						SSCP.prefService.delValue(list.id, key);
    						list.removeChild(style);
    						break;
    					}
    				}
    			}
    		}
    	}
	},
	
	/* Deletes all styles from the cache (selects entire list and asks deletion)
	 * @param id The styles listbox id from which the user asked a deletion
	 */
	delAllStyles: function(id) {
	   var list = document.getElementById(id);
	   if (list) {
    		list.selectAll();
    		this.delStyle(id);
    	}
	}
};
