// ------------------------------------------------------------------------------------------------

/*
    This file is part of Style Sheet Chooser Plus.

    Style Sheet Chooser Plus is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Style Sheet Chooser Plus is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Style Sheet Chooser Plus; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// ------------------------------------------------------------------------------------------------
//
// Style Sheet Chooser Plus
//
// Copyright 2004-2006, OLab (http://www.olab.info/)
// Thanks to Rob Tougher (http://www.robtougher.com) for the original extension
// namely Stylesheet Chooser, on which this one is partly based
//
// This extension persists your choices for alternate stylesheets on
// a per-domain basis. For example, if you browse to http://domain.com/index1.html,
// choose the alternate stylesheet "Alt", and browse to http://domain.com/index2.html,
// the alternate stylesheet "Alt" will be used on the second page, index2.html.
// Furthermore, an icon in the status bar helps you to select it.
//
// Choices are persisted between window instances, so your choices will be remembered
// if you close Firefox and restart it.
//
// ------------------------------------------------------------------------------------------------

function SSCPSQLite() {
	// Parameters
	this.optionsTable = 'options';
	this.authorStylesTable = 'authorStyles';

	var file = Components.classes["@mozilla.org/file/directory_service;1"]
	                     .getService(Components.interfaces.nsIProperties)
	                     .get("ProfD", Components.interfaces.nsIFile);
	file.append("sscp.sqlite");
	
	this.dbFile = file;
	
	var storageService = Components.classes["@mozilla.org/storage/service;1"]
	                        .getService(Components.interfaces.mozIStorageService);
	var mDBConn = storageService.openDatabase(file);
	
	this.db = mDBConn;
	
	// Creating tables if necessary
	this.initDB();
}

/* Database tables creation */
SSCPSQLite.prototype.initDB = function () {
	if (this.db) {
		// Options table
		var createSt = "CREATE TABLE IF NOT EXISTS " + this.optionsTable + " (key TEXT, value TEXT, PRIMARY KEY (key))";
		var statement = this.db.createStatement(createSt);
		statement.execute();
		statement.reset();

		// Author styles table
		var createSt = "CREATE TABLE IF NOT EXISTS " + this.authorStylesTable + " (key TEXT, value TEXT, PRIMARY KEY (key))";
		var statement = this.db.createStatement(createSt);
		statement.execute();
		statement.reset();
	}
};

/* Gets a value from a table
 * @param string type Table name in which we want the value
 * @param string key Unique identifier of the value
 * @param string defValue The value returned if the key wasn't found
 * @return string The value corresponding to key
 */
SSCPSQLite.prototype.getValue = function (type, key, defValue) {
	var query = "SELECT value FROM " + type + " WHERE key = '" + key + "'";
	var statement = this.db.createStatement(query);
	var val = null;
	if (statement.executeStep()) {
		val = statement.getUTF8String(0);
		if (val == 'false')
			val = false;
		else if (val == 'true')
			val = true;
	} else {
		val = defValue;
	}
	statement.reset();
	return val;
};

/* Sets a value in a table
 * @param string type Table name in which we want to set the value
 * @param string key Unique identifier of the value
 * @param string value The value
 */
SSCPSQLite.prototype.setValue = function (type, key, value) {
	var query = "INSERT OR REPLACE INTO " + type + " VALUES ('" + key + "', '" + value + "')";
	var statement = this.db.createStatement(query);
	statement.execute();
	statement.reset();
};

/* Deletes a value from a table
 * @param string type Table name in which we want to delete the value
 * @param string key Unique identifier of the value
 */
SSCPSQLite.prototype.delValue = function (type, key) {
	var query = "DELETE FROM " + type + " WHERE key='" + key + "'";
	var statement = this.db.createStatement(query);
	statement.execute();
	statement.reset();
};

/* Gets all values from a table
 * @param string type Table name in which we want the values
 * @return array([string,string]) Array of key,value pairs
 */
SSCPSQLite.prototype.getAllValues = function (type) {
	var query = "SELECT * FROM " + type + " ORDER BY key";
	var statement = this.db.createStatement(query);
	var values = new Array();
	while (statement.executeStep()) {
		var key = statement.getUTF8String(0);
		var val = statement.getUTF8String(1);
		if (val == 'false')
			val = false;
		else if (val == 'true')
			val = true;
		values.push(new Array(key, val));
	}
	statement.reset();
	return values;
};
