/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is ColorZilla Mozilla Extension.
 * The Initial Developer of the Original Code is Alex Sirota <alex@elbrus.com>. 
 * Portions created by Alex Sirota are Copyright (C) 2004 Alex Sirota. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var gCZClickListener;

function ColorZillaClickListener() 
{
}

ColorZillaClickListener.prototype = 
{
  mbListening: false,
  mTargetDocs: null,

  startClickListener: function()
  {
    this.mbListening = true;
    this.mTargetDocs = this.getAllDocuments();

    for (var i = 0; i < this.mTargetDocs.length; ++i) {
      this.mTargetDocs[i].addEventListener("mousedown", CZMouseDownListener, true);
      this.mTargetDocs[i].addEventListener("mouseup", CZEventCanceller, true);
      this.mTargetDocs[i].addEventListener("click", CZListenerRemover, true);
    }
  },

  stopClickListener: function(aTarget)
  {
    this.mbListening = false;
    czToggleColorMonitoring(true);
  },

  removeClickListeners: function()
  {
    if (this.mTargetDocs == null) {
	// we never installed the listeners
	return;
    }

    for (var i = 0; i < this.mTargetDocs.length; ++i) {
      this.mTargetDocs[i].removeEventListener("mousedown", CZMouseDownListener, true);
      this.mTargetDocs[i].removeEventListener("mouseup", CZEventCanceller, true);
      this.mTargetDocs[i].removeEventListener("click", CZListenerRemover, true);
    }
  },

  // currently, returns only the main window
  getAllDocuments: function()
  {
	return [window];
  },
}  


// Listener Objects
var CZMouseDownListener = {
  handleEvent: function(aEvent)
  {
    var target = aEvent.target;
    if (aEvent.type == "mousedown") {
      aEvent.stopPropagation();
      aEvent.preventDefault();
      if (aEvent.shiftKey && !aEvent.altKey) {
		// shift pressed (no alt), add to history, go on listening
		gCZCustomPaletteUI.AddColorToHistory(gCZSelectedColor, czGetCurrentElementLongName());
      } else if (aEvent.shiftKey && aEvent.altKey) {
		// shift and alt pressed, panning, just return
		return;
      } else {
		// no shift, no alt, stop listening
		gCZClickListener.stopClickListener(target);
      }
    }
  }
};

var CZEventCanceller = {
  handleEvent: function(aEvent)
  {
    aEvent.stopPropagation();
    aEvent.preventDefault();
  }
};

var CZListenerRemover = {
  handleEvent: function(aEvent)
  {
      if (aEvent.type == "click") {
        aEvent.stopPropagation();
        aEvent.preventDefault();
      }
      if (!gCZClickListener.mbListening) {
        gCZClickListener.removeClickListeners();
      }
  }
};


gCZClickListener = new ColorZillaClickListener();

