/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is ColorZilla Mozilla Extension.
 * The Initial Developer of the Original Code is Alex Sirota <alex@elbrus.com>. 
 * Portions created by Alex Sirota are Copyright (C) 2004 Alex Sirota. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function czColorPickerGetEntryboxElem(name) {
	return document.getElementById('colorzilla-textbox-' + name);
}

function czUpdateColorPicker(color, bUpdatePalette, bNoHSVUpdate, colorNameOverride) {
	gCZCurrentColor = color;

	var colorPanel = document.getElementById('colorzilla-picker-current-color');
	var rgbAttribute = czColToRGBAttribute(color);
      	colorPanel.style.backgroundColor = rgbAttribute;

	var red   = czGetRValue(color);
	var green = czGetGValue(color);
	var blue  = czGetBValue(color);
	czColorPickerGetEntryboxElem('red').value = red;
	czColorPickerGetEntryboxElem('green').value = green;
	czColorPickerGetEntryboxElem('blue').value = blue;

	if (!bNoHSVUpdate) {
		var hsv = czRGBToHSV(red, green, blue);
		czColorPickerGetEntryboxElem('hue').value = hsv.h;
		czColorPickerGetEntryboxElem('sat').value = hsv.s;
		czColorPickerGetEntryboxElem('val').value = hsv.v;
	}

	var colorHex = czColToRGBHexaAttribute(color);
	czColorPickerGetEntryboxElem('hex').value = colorHex;

	
	czColorPickerGetEntryboxElem('css').value = rgbAttribute;

	var colorName = colorNameOverride ? colorNameOverride : 
				czHexaAttributeToPredefinedColor(colorHex);
	var colorNameElem = czColorPickerGetEntryboxElem('color-name');
	if (colorName != null) {
		colorNameElem.value = colorName;
	} else {
		colorNameElem.value = '';
	}

	// only update the picker palette if it is visible
	if (bUpdatePalette && (czGetCurrentTab() == 'picker')) {
		czCreateColorPalette(false);
		czCreateColorElementSlider(false);
	}
}

function czColorPickerRGBChanged(event) {
	var entryBox = event.target;
	entryBox.value = czFixByteValue(entryBox.value);

	var red   = czColorPickerGetEntryboxElem('red').value;
	var green = czColorPickerGetEntryboxElem('green').value;
	var blue  = czColorPickerGetEntryboxElem('blue').value;

	var newColor = czRGBToColor(red, green, blue);
	czUpdateColorPicker(newColor, true);
}

function czColorPickerHSVChanged(event) {
	var entryBox = event.target;
	entryBox.value = czFixByteValue(entryBox.value);

	var hue   = czColorPickerGetEntryboxElem('hue').value;
	var sat = czColorPickerGetEntryboxElem('sat').value;
	var val  = czColorPickerGetEntryboxElem('val').value;

	var rgb = czHSVToRGB(hue, sat, val);
	var newColor = czRGBToColor(rgb.r, rgb.g, rgb.b);
	czUpdateColorPicker(newColor, true, true);
}

function czColorPickerHexChanged(event) {
	var entryBox = event.target;
	entryBox.value = czFixHexValue(entryBox.value);

	var newColor = czRGBHexaAttributeToCol(entryBox.value);
	czUpdateColorPicker(newColor, true);
}

function czMarkClickTarget(color, element, prevElement) {
	// clear the mark from the previous element
	if (prevElement != null) {
		prevElement.className = '';
	}

	if (element == null) {
		// no mark target, return
		return;
	}

	if (czGetColorLightness(color) > 128) {
		element.className = "colorzilla-selected-dark";
	} else {
		element.className = "colorzilla-selected-bright";
	}
}

function czColorPaletteClicked(event) {
//	var newColor = czGetScreenColor(event.screenX, event.screenY);
	var newColor = czRGBAttributeToCol(event.target.style.backgroundColor);
	czUpdateColorPicker(newColor, false);

	// mark the clicked item:
	czMarkClickTarget(newColor, event.target, gOldPaletteTarget);
	gOldPaletteTarget = event.target;

	// we don't want to update the whole palette (hence the 'false' arg above)
	// we do want to update the slider:
	czCreateColorElementSlider(false);
}

function czColorElementSliderClicked(event) {
	//var newColor = czGetScreenColor(event.screenX, event.screenY);
	var newColor = czRGBAttributeToCol(event.target.style.backgroundColor);
	var r = czGetRValue(newColor);
	var g = czGetGValue(newColor);
	var b = czGetBValue(newColor);

	var paletteType = document.getElementById('colorzilla-picker-palette-type').selectedItem.value;
	var bHSVChanged = false;
	switch (paletteType) {
		case 'hue':
			var hsv = czRGBToHSV(r, g, b);
			czColorPickerGetEntryboxElem('hue').value = hsv.h;
			bHSVChanged = true;		
			break;
		case 'sat':
			var hsv = czRGBToHSV(r, g, b);
			czColorPickerGetEntryboxElem('sat').value = hsv.s;
			bHSVChanged = true;
			break;
		case 'val':
			var hsv = czRGBToHSV(r, g, b);
			czColorPickerGetEntryboxElem('val').value = hsv.v;
			bHSVChanged = true;
			break;
		case 'red':
			czColorPickerGetEntryboxElem('red').value = r;
			break;
		case 'green':
			czColorPickerGetEntryboxElem('green').value = g;
			break;
		case 'blue':
			czColorPickerGetEntryboxElem('blue').value = b;
			break;
		default:
			dump("Error, unexpected palette type: " + paletteType + "\n");
			return false;
	}

	var newColor;
	if (bHSVChanged) {
		var hue = czColorPickerGetEntryboxElem('hue').value;
		var sat = czColorPickerGetEntryboxElem('sat').value;
		var val = czColorPickerGetEntryboxElem('val').value;
		var rgb = czHSVToRGB(hue, sat, val);
		newColor = czRGBToColor(rgb.r, rgb.g, rgb.b);
	} else {
		var red = czColorPickerGetEntryboxElem('red').value;
		var green = czColorPickerGetEntryboxElem('green').value;
		var blue = czColorPickerGetEntryboxElem('blue').value;
		var newColor = czRGBToColor(red, green, blue);
	}

	czUpdateColorPicker(newColor, true, bHSVChanged);

	// mark the clicked item:
	czMarkClickTarget(newColor, event.target, gOldSliderTarget);
	gOldSliderTarget = event.target;
}


// Palette building functions
function czBuildPaletteColorByHue(hue, x, y) {
	var rgb = czHSVToRGB(hue, x, 255 - y);
	return czRGBToColor(rgb.r, rgb.g, rgb.b);
}

function czBuildPaletteColorBySat(sat, x, y) {
	var rgb = czHSVToRGB(x, sat, 255 - y);
	return czRGBToColor(rgb.r, rgb.g, rgb.b);
}

function czBuildPaletteColorByVal(val, x, y) {
	var rgb = czHSVToRGB(x, 255 - y, val);
	return czRGBToColor(rgb.r, rgb.g, rgb.b);
}

function czBuildPaletteColorByRed(red, x, y) {
	return czRGBToColor(red, x, 255-y);
}

function czBuildPaletteColorByGreen(green, x, y) {
	return czRGBToColor(x, green, 255-y);
}

function czBuildPaletteColorByBlue(blue, x, y) {
	return czRGBToColor(x, 255-y, blue);
}
// End palette building functions

function czCreateColorPalette(bCreate) {
	var paletteContainer;
	
	if (bCreate) {
		paletteContainer = document.createElement('stack');
		paletteContainer.setAttribute("id", "colorzilla-picker-color-palette");
		paletteContainer.setAttribute("onclick", "czColorPaletteClicked(event)");
		paletteContainer.setAttribute('width', 256 + quantizationAmount);
		paletteContainer.setAttribute('height', 256 + quantizationAmount);
//		window.sizeToContent();
	} else {
		paletteContainer = document.getElementById('colorzilla-picker-color-palette');
	}

	var quantizationAmount = gCZPaletteSize*1;

	var hue = czColorPickerGetEntryboxElem('hue').value;
	var sat = czColorPickerGetEntryboxElem('sat').value;
	var val = czColorPickerGetEntryboxElem('val').value;
	var red = czColorPickerGetEntryboxElem('red').value;
	var green = czColorPickerGetEntryboxElem('green').value;
	var blue = czColorPickerGetEntryboxElem('blue').value;

	var paletteType = document.getElementById('colorzilla-picker-palette-type').selectedItem.value;
	var buildPaletteFunction, constVal;
	switch (paletteType) {
		case 'hue':
			buildPaletteFunction = czBuildPaletteColorByHue;
			constVal = hue;
			break;
		case 'sat':
			buildPaletteFunction = czBuildPaletteColorBySat;
			constVal = sat;
			break;
		case 'val':
			buildPaletteFunction = czBuildPaletteColorByVal;
			constVal = val;
			break;
		case 'red':
			buildPaletteFunction = czBuildPaletteColorByRed;
			constVal = red;
			break;
		case 'green':
			buildPaletteFunction = czBuildPaletteColorByGreen;
			constVal = green;
			break;
		case 'blue':
			buildPaletteFunction = czBuildPaletteColorByBlue;
			constVal = blue;
			break;
		default:
			dump("Error, unexpected palette type: " + paletteType + "\n");
			return false;
	}
					
	var currentElem, prevElem;
	if (!bCreate) {
		currentElem = paletteContainer.firstChild;
	}

	var bMarked = false;
	var minMarkedDist = 256*3;
	var markTarget = null;
	var markColor = null;
	for (var x=0; x < 263; x += quantizationAmount) {
		for (var y=0; y < 263; y += quantizationAmount) {
			var newColor      = buildPaletteFunction(constVal, Math.min(x, 255), Math.min(y, 255));
			var newBackground = czColToRGBAttribute(newColor);

			var dR = Math.abs(czGetRValue(newColor) - red);
			var dG = Math.abs(czGetGValue(newColor) - green);
			var dB = Math.abs(czGetBValue(newColor) - blue);
			var totalDist = dR + dG + dB;
	
			var bMark = false;
			if (
				(dR < quantizationAmount) &&
				(dG < quantizationAmount) &&
				(dB < quantizationAmount) && 
				(totalDist < minMarkedDist)
			   ) {
				bMark = true;	
				minMarkedDist = totalDist;
			}

			if (bCreate) {
				var newElem = document.createElement('box');
				newElem.setAttribute('width', quantizationAmount);
				newElem.setAttribute('height', quantizationAmount);
				newElem.setAttribute('left', x);
				newElem.setAttribute('top', y);
				newElem.style.backgroundColor = newBackground;

				paletteContainer.appendChild(newElem);
				prevElem = newElem;
			} else {
				currentElem.style.backgroundColor = newBackground;
				prevElem = currentElem;
				currentElem = currentElem.nextSibling;
			}

			if (bMark) {
				markTarget = prevElem;
				markColor  = newColor;
				bMarked = true;
			}
		}
	}

	if (!bMarked) {
		// if haven't marked anything, clear the prev mark
		czMarkClickTarget(null, null, gOldPaletteTarget);
		gOldPaletteTarget = null;
	} else {
		// mark the specified element
		if ((markTarget != null) && (markColor != null)) {
			czMarkClickTarget(markColor, markTarget, gOldPaletteTarget);
			gOldPaletteTarget = markTarget;
		}
	}

	if (bCreate) {
		// insert the final element into the panel
		var panel = document.getElementById('colorzilla-picker-color-palette-panel');
		if (panel.firstChild) {
			panel.insertBefore(paletteContainer, panel.firstChild);
		} else {
			panel.appendChild(paletteContainer);
		}
	}
}


// Slider building functions
function czBuildSliderByHue(sat, val, y) {
	//var rgb = czHSVToRGB(255 - y, sat, val);
	var rgb = czHSVToRGB(255 - y, 255, 255);
	return czRGBToColor(rgb.r, rgb.g, rgb.b);
}

function czBuildSliderBySat(hue, val, y) {
	var rgb = czHSVToRGB(hue, 255 - y, val);
	return czRGBToColor(rgb.r, rgb.g, rgb.b);
}

function czBuildSliderByVal(hue, sat, y) {
	var rgb = czHSVToRGB(hue, sat, 255 - y);
	return czRGBToColor(rgb.r, rgb.g, rgb.b);
}

function czBuildSliderByRed(green, blue, y) {
	return czRGBToColor(255-y, green, blue);
}

function czBuildSliderByGreen(red, blue, y) {
	return czRGBToColor(red, 255-y, blue);
}

function czBuildSliderByBlue(red, green, y) {
	return czRGBToColor(red, green, 255-y);
}
// End slider building functions

function czCreateColorElementSlider(bCreate) {
	var sliderContainer = document.getElementById('colorzilla-picker-color-element-slider');
	var width = sliderContainer.width;
	var quantizationAmount = 4;

	var hue = czColorPickerGetEntryboxElem('hue').value;
	var sat = czColorPickerGetEntryboxElem('sat').value;
	var val = czColorPickerGetEntryboxElem('val').value;
	var red = czColorPickerGetEntryboxElem('red').value;
	var green = czColorPickerGetEntryboxElem('green').value;
	var blue = czColorPickerGetEntryboxElem('blue').value;

	var paletteType = document.getElementById('colorzilla-picker-palette-type').selectedItem.value;
	var buildSliderFunction, constVal1, constVal2;
	var currentVal;
	switch (paletteType) {
		case 'hue':
			buildSliderFunction = czBuildSliderByHue;
			constVal1 = sat;
			constVal2 = val;
			currentVal = hue;
			break;
		case 'sat':
			buildSliderFunction = czBuildSliderBySat;
			constVal1 = hue;
			constVal2 = val;
			currentVal = sat;
			break;
		case 'val':
			buildSliderFunction = czBuildSliderByVal;
			constVal1 = hue;
			constVal2 = sat;
			currentVal = val;
			break;
		case 'red':
			buildSliderFunction = czBuildSliderByRed;
			constVal1 = green;
			constVal2 = blue;
			currentVal = red;
			break;
		case 'green':
			buildSliderFunction = czBuildSliderByGreen;
			constVal1 = red;
			constVal2 = blue;
			currentVal = green;
			break;
		case 'blue':
			buildSliderFunction = czBuildSliderByBlue;
			constVal1 = red;
			constVal2 = green;
			currentVal = blue;
			break;
		default:
			dump("Error, unexpected palette type: " + paletteType + "\n");
			return false;
	}
					

	var currentElem;
	if (!bCreate) {
		currentElem = sliderContainer.firstChild;
	}
	var bMark = false;
	var bMarked = false;
	var prevElem;
	for (var y=0; y <= 260; y += quantizationAmount) {
		var sliderVal = Math.min(y, 255);
		var newColor = buildSliderFunction(constVal1, constVal2, sliderVal);
		var newBackground = czColToRGBAttribute(newColor);

		if (Math.abs((255 - sliderVal) - currentVal) < quantizationAmount) {
			bMark = true;
		}

		if (bCreate) {
			var newElem = document.createElement('box');
			newElem.setAttribute('width', width);
			newElem.setAttribute('height', quantizationAmount);
			newElem.setAttribute('top', y);
			newElem.style.backgroundColor = newBackground;
			sliderContainer.appendChild(newElem);
			prevElem = newElem;
		} else {
			currentElem.style.backgroundColor = newBackground;
			prevElem = currentElem;
			currentElem = currentElem.nextSibling;
		}

		if (bMark && !bMarked) {
			czMarkClickTarget(newColor, prevElem, gOldSliderTarget);
			gOldSliderTarget = prevElem;
			bMarked = true;
		}
	}
}

function czCreateColorPalettes(bCreate) {
	// only update the picker palette if it is visible
	if (!bCreate && (czGetCurrentTab() != 'picker')) {
		return;
 	}

	// save the palette type:
	var radioGroup = document.getElementById('colorzilla-picker-palette-type');
	czWritePref("palette-type-item-id", radioGroup.selectedItem.id);

	czCreateColorPalette(bCreate);
	czCreateColorElementSlider(bCreate);
}

function czColorPickerOK() {
	try {
		if (gCZOKCallback) {
			gCZOKCallback(gCZCurrentColor);
		}
	} catch(e) {
		dump("Warning: erorr updating current color: " + e + "\n");
	}
	window.close();
}

function czGetCurrentTab() {
	var tabs = document.getElementById('colorzilla-colorpicker-tabs');
	return tabs.selectedItem.getAttribute('value');
}

function czSetInitialTab() {
	var lastSelectedTabIndex = czReadPref("last-selected-colorpicker-tab", -1);
	if (lastSelectedTabIndex != -1) {
		var tabs = document.getElementById('colorzilla-colorpicker-tabs');
		if (tabs.selectedIndex != lastSelectedTabIndex) {
			tabs.selectedIndex = lastSelectedTabIndex;
		}
	}
}

function czTabSelected(event) {
	var selectedTab = event.target;
	var selectedTabIndex = selectedTab.getAttribute("tabindex");	

	czWritePref("last-selected-colorpicker-tab", selectedTabIndex);

	// if our current tab is the color picker, update the palette and the slider
	var currentTab = czGetCurrentTab();
	if (currentTab == 'picker') {
		czCreateColorPalettes(false);
	} else if (currentTab == 'palettes') {
		gCZCustomPaletteUI.ReloadCurrentPalette();
		gCZCustomPaletteUI.ClearPaletteMarkers();	
	}
}

function czInitColorPicker() {
  	var color = null;

	if (window.arguments) {
		color = window.arguments[0];
		gCZOKCallback = window.arguments[1];
	} else {
		// we were not called as a dialog, hide buttons
		var okButton = document.getElementById('colorzilla-picker-ok-button');
		okButton.setAttribute('hidden', 'true');
		var cancelButton = document.getElementById('colorzilla-picker-cancel-button');
		cancelButton.setAttribute('hidden', 'true');
	}

	if (color == null) {
		color = czRGBToColor(255, 255, 255);
	}
	var radioGroup = document.getElementById('colorzilla-picker-palette-type');
	var selectedItemID = czReadPref("palette-type-item-id", "colorzilla-radio-hue");
        radioGroup.selectedItem = document.getElementById(selectedItemID);

	gCZPaletteSize = czReadPref("palette-size", 16);
	czUpdateColorPicker(color, false);

	czCreateColorPalette(true);
	czCreateColorElementSlider(true);

	czSetInitialTab();
	
	try {
		gCZCustomPaletteUI.Init();
	} catch(e) { 
		dump("Error initializing custom palettes: " + e + "\n"); 
	}
}

// the current color
var gCZCurrentColor;

// palette size
var gCZPaletteSize;

// callback function (when OK is pressed):
var gCZOKCallback = null;
// the previous element selected in the palette/slider:
var gOldPaletteTarget = null;
var gOldSliderTarget = null;
