/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is ColorZilla Mozilla Extension.
 * The Initial Developer of the Original Code is Alex Sirota <alex@elbrus.com>. 
 * Portions created by Alex Sirota are Copyright (C) 2004 Alex Sirota. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

const COLORZILLA_COPY_RGB 		= 0	// rgb(255, 255, 255)
const COLORZILLA_COPY_RGB_PERC 		= 1	// rgb(100%, 100%, 100%)
const COLORZILLA_COPY_HEX 		= 2	// #FFFFFF
const COLORZILLA_COPY_HEX_SIMPLE 	= 3	// FFFFFF

function czShowHideMainPanel(bShow) {
	var mainPanel = document.getElementById('colorzilla-statusbar-panel');

	var oldCollapsed = mainPanel.getAttribute('collapsed');

	if (bShow) {
		mainPanel.setAttribute('collapsed', false);
	} else {
		// do not hide when listening
		if (!gbCZMonitorCurrentColor) {
			var message = czGetLocaleString('youCanPress') + 
						" " + gShorcutKeyName + " " + 
						czGetLocaleString('toReveal') + " ColorZilla";
			czSetStatusText(message);
			setTimeout(czSetStatusText, 4000, '');
			mainPanel.setAttribute('collapsed', true);
		}
	}
	return oldCollapsed;
}

function czSetStatusText(str) {
	var statusTextFld = document.getElementById("statusbar-display");
        statusTextFld.label = str;
}

function czGetStatusText() {
	var statusTextFld = document.getElementById("statusbar-display");
        return statusTextFld.label;
}

function czSetStatusBarFormat(newFormat) {
	gCZStatusBarFormat = newFormat;
	czWritePref("status-bar-format", gCZStatusBarFormat);
}

function czChangeStatusBarFormat() {

	window.openDialog('chrome://colorzilla/content/colorzillaStatusbarFormat.xul',
							'',
							'chrome,centerscreen,modal',
							gCZStatusBarFormat);
}

function czGetNodeCSSDimensions(node) {
	var width = 0;
	var height = 0;
	if (node != null) {
		width = node.ownerDocument.defaultView.getComputedStyle(node, "").
				getPropertyValue("width");
		if (width != 'auto') {
	    		width = width.substr(0, width.length - 2);
		} else {
			width = "auto-";
		}
		height = node.ownerDocument.defaultView.getComputedStyle(node, "").
				getPropertyValue("height");
		if (height != 'auto') {
			height = height.substr(0, height.length - 2);
		} else {
			height = '-auto';
		}
	}

	return width + "x" + height;
}

function czGetNodeRelativeCoordinates(node, clientX, clientY) {
	var x;
	var y;

	if ("boxObject" in node) { 
		// xul elements
		x = node.boxObject.x;		
		y = node.boxObject.y;
	} else {
	 	// html elements

		// get the coordinates
		x = 0;
		y = 0;
		var currNode = node;
		while (currNode.offsetParent) {
			x += currNode.offsetLeft;
			y += currNode.offsetTop;
			currNode = currNode.offsetParent;
		}

		var doc = node.ownerDocument;
		if (doc) {
			// subtract the scroll position
			if (doc.documentElement && doc.documentElement.scrollTop) {
				y -= doc.documentElement.scrollTop;
				x -= doc.documentElement.scrollLeft;
			} else if (doc.body && doc.body.scrollTop) {
				y -= doc.body.scrollTop;
				x -= doc.body.scrollLeft;
			}
		}
    	}


	var offsetX = (clientX - x);
	var offsetY = (clientY - y);

	return "X: " + offsetX + ", Y: " + offsetY;
}

function czGetNodeBoxDimensions(node) {
	var boxWidth;
	var boxHeight;
	
	if ("boxObject" in node) { 
		// xul
		boxWidth = node.boxObject.width;
		boxHeight = node.boxObject.height;
	} else {
		// html
		boxWidth = node.offsetWidth;
		boxHeight = node.offsetHeight;
	}

	return boxWidth + "x" + boxHeight;    	
}

function czBuildStatusBarText() {
	var col = gCZSelectedColor;

	// RGB
	var r = czGetRValue(col);
	var g = czGetGValue(col);
	var b = czGetBValue(col);
	var rgb = "R: " + r + ", G: " + g + ", B: " + b;
    var rgbPerc = "R: " + czIntToPercent(r) + "%, G: " + czIntToPercent(g) + "%, B: " + czIntToPercent(b) + "%";    

	// HSV
	var hsv = czRGBToHSV(r, g, b);
	hsv = "H: " + hsv.h + ", S: " + hsv.s + ", V: " + hsv.v;

	// Hex
	var hex = czColToRGBHexaAttribute(col);

	// Element tag, id, class
	var elemInfo = czGetCurrentElementLongName();

	// Distance from previous point
	var width;
	var height;
	if (gCZPrevSelectedX != null) {
		width  = Math.abs(gCZPrevSelectedX - gCZSelectedX);
		height = Math.abs(gCZPrevSelectedY - gCZSelectedY);
	} else {
		width = 0;
		height = 0;
	}
	var distance = "\u0394X: " + width + ", \u0394Y: " + height;

	// Size of the current element
	var node = gCZSelectedElement;
	var cssSize = '';
	var boxSize = '';
	if (node != null) {
		boxSize = czGetNodeBoxDimensions(node);
		cssSize = czGetNodeCSSDimensions(node);
	}

	// the relative coordinates of the sampled point relative to the node 
	var relativePos = ' - '; 
	if ((node != null) && (gCZSelectedClientX != null) && (gCZSelectedClientY != null)) {
		relativePos = czGetNodeRelativeCoordinates(node, gCZSelectedClientX, gCZSelectedClientY);
	}

	var fullURL = '';
	var pathURL = '';
	var fileURL = '';
	if (gCZSelectedElement != '') {
		fullURL = gCZSelectedElement.ownerDocument.location.href;
		pathURL = gCZSelectedElement.ownerDocument.location.pathname;
		fileURL = pathURL.substring((pathURL.lastIndexOf("/")) + 1);
	}

	// format the stings
	var finalString = gCZStatusBarFormat;

	// check if the format string contains dom-path
	// to avoid calculating it when not needed
	if (finalString.indexOf('%dom-path%') != -1) {
		var domPath = czGetElementTreePath(gCZSelectedElement, false);
		finalString = finalString.replace(/%dom-path%/g, domPath);
	}

	if (finalString.indexOf('%dom-path-reversed%') != -1) {
		var domPath = czGetElementTreePath(gCZSelectedElement, true);
		finalString = finalString.replace(/%dom-path-reversed%/g, domPath);
	}

	finalString = finalString.replace(/%rgb%/g, rgb);
	finalString = finalString.replace(/%rgb-perc%/g, rgbPerc);
	finalString = finalString.replace(/%hsv%/g, hsv);
	finalString = finalString.replace(/%hex%/g, hex);
	finalString = finalString.replace(/%element%/g, elemInfo);
	finalString = finalString.replace(/%dist%/g, distance);
	finalString = finalString.replace(/%css-size%/g, cssSize);
	finalString = finalString.replace(/%box-size%/g, boxSize);
	finalString = finalString.replace(/%rel-pos%/g, relativePos);
	finalString = finalString.replace(/%url-full%/g, fullURL);
	finalString = finalString.replace(/%url-path%/g, pathURL);
	finalString = finalString.replace(/%url-file%/g, fileURL);

	return finalString;
//	return rgb + " | " + hsv + " | " + hex + " | " + elemInfo + " | " + distance + " | " + elemSize;
}

function czUpdateStatusPanel(str) {
	czSetStatusText(str);
}

function czUpdateColorPanel(bNoStatus) {
	var col = gCZSelectedColor;
	var colorDisplayElement = document.getElementById("colorzilla-status-bar-color");
	var rgbAttribute = czColToRGBAttribute(col);
      	colorDisplayElement.style.backgroundColor = rgbAttribute + "! important";

	var description = czBuildStatusBarText();
   	colorDisplayElement.setAttribute("tooltiptext", description);
    gCZLastStatusValue = description;

	if (!bNoStatus) {
		czUpdateStatusPanel(description);
	}
}

function czCopyColorValue(elem) {
	var value = elem.getAttribute('value');
	czCopyStringToClipboard(value);
}

function czUpdateColorMenuEntries() {
	var getStr = czGetLocaleString('copy');

	var getRGBElem = document.getElementById('colorzilla-menu-copy-rgb');
	var rgbAttribute = czColToRGBAttribute(gCZSelectedColor);
	getRGBElem.setAttribute('hidden', "false");
	getRGBElem.setAttribute('value', rgbAttribute);
	getRGBElem.setAttribute('label', getStr + " \"" + rgbAttribute + "\"");

	var getRGBPercentageElem = document.getElementById('colorzilla-menu-copy-rgb-percentage');
	var rgbPercentageAttribute = czColToRGBPercentageAttribute(gCZSelectedColor);
	getRGBPercentageElem.setAttribute('hidden', "false");
	getRGBPercentageElem.setAttribute('value', rgbPercentageAttribute);
	getRGBPercentageElem.setAttribute('label', getStr + " \"" + rgbPercentageAttribute + "\"");

	var getRGBHexElem = document.getElementById('colorzilla-menu-copy-rgb-hex');
	var rgbHexAttribute = czColToRGBHexaAttribute(gCZSelectedColor);
	getRGBHexElem.setAttribute('hidden', "false");
	getRGBHexElem.setAttribute('value', rgbHexAttribute);
	getRGBHexElem.setAttribute('label', getStr + " \"" + rgbHexAttribute + "\"");

	var getRGBHexNoNumberElem = document.getElementById('colorzilla-menu-copy-rgb-hex-no-number-sign');
	var rgbHexNoNumberAttribute = rgbHexAttribute.substring(1);
	getRGBHexNoNumberElem.setAttribute('hidden', "false");
	getRGBHexNoNumberElem.setAttribute('value', rgbHexNoNumberAttribute);
	getRGBHexNoNumberElem.setAttribute('label', getStr + " \"" + rgbHexNoNumberAttribute + "\"");

	var getPredefinedColorElem = document.getElementById('colorzilla-menu-copy-predefined-color');
	var predefinedColor = czHexaAttributeToPredefinedColor(rgbHexAttribute);
	if (predefinedColor != null) {
		getPredefinedColorElem.setAttribute('hidden', "false");
		getPredefinedColorElem.setAttribute('value', predefinedColor);
		getPredefinedColorElem.setAttribute('label', getStr + " \"" + predefinedColor + "\"");
	} else {
		getPredefinedColorElem.setAttribute('hidden', "true");
		getPredefinedColorElem.setAttribute('value', "");
		getPredefinedColorElem.setAttribute('label', "");
	}

	var getLastStatusValueElem = document.getElementById('colorzilla-menu-copy-last-status-value');
	getLastStatusValueElem.setAttribute('hidden', "false");
	getLastStatusValueElem.setAttribute('value', gCZLastStatusValue);
	getLastStatusValueElem.setAttribute('label', getStr + " \"" + szClipString(gCZLastStatusValue, 15) + "\"");


	var addToFavoritesElem = document.getElementById('colorzilla-menu-add-to-favorites');
	addToFavoritesElem.setAttribute('hidden', "false");
	addToFavoritesElem = document.getElementById('colorzilla-menu-add-to-favorites-context');
	addToFavoritesElem.setAttribute('hidden', "false");

	var getRGBSeparatorElem = document.getElementById('colorzilla-menu-copy-separator');
	getRGBSeparatorElem.setAttribute('hidden', "false");

}

function czShowStatusLineHelp(bRestore) {
	if (gCZComponent == null) {
		// do not show a message about eyedropper if there is no eyedropper
		return;
	}

	var noHelpElem = document.getElementById('colorzilla-menu-no-statusbar-help');
	if (noHelpElem.getAttribute('checked') == 'true') {
		return;
	}


	if (!bRestore) {
                var helpString = czGetLocaleString('press') + " " + 
					gShorcutKeyName + " " + 	
					czGetLocaleString('toStartTheEyedropperMode');
		var currentStatus = czGetStatusText();
		if (currentStatus != helpString) {
			gCZSavedStatus = currentStatus;
		}
		czSetStatusText(helpString);
	} else {
		czSetStatusText(gCZSavedStatus);
	}
}

function czZoomElements(node, scaleFactor) {
	var childNodes = node.childNodes;

	var childNode = null;
	for (var i = 0; i < childNodes.length; i++) {
		childNode = childNodes[i];
		
		if (childNode.nodeType == 1) {
			czZoomElements(childNode, scaleFactor);
		}
    	}

	// for frames:
	if (node.contentDocument) {
			czZoomElements(node.contentDocument, scaleFactor);
	}

	try {
		if (node.localName == "IMG") {
			var width = node.ownerDocument.defaultView.getComputedStyle(node, "").getPropertyValue("width");
	       		width = width.substr(0, width.length - 2);
			var height = node.ownerDocument.defaultView.getComputedStyle(node, "").getPropertyValue("height");
	       		height = height.substr(0, height.length - 2);
			if ((width != 'au') && (height != 'au')) {
			    var origWidth  = node.origWidth;
			    var origHeight = node.origHeight;
			    if (origWidth && origHeight) {
				width = origWidth;
				height = origHeight;
			    } else {
				node.origWidth = width;
				node.origHeight = height;
			    }
        		    node.setAttribute("width", width * scaleFactor);
	       		    node.setAttribute("height", height * scaleFactor);
			}
		}
	} catch (e) {}

	//ZoomManager.prototype.getInstance().textZoom = 200;
	try {
		var fontSize = node.ownerDocument.defaultView.getComputedStyle(node, "").getPropertyValue("font-size");
                var origFontSize = node.origFontSize;
		if (origFontSize) {
			fontSize = origFontSize;
		} else {
			node.origFontSize = fontSize;
		}
       		fontSize = fontSize.substr(0, fontSize.length - 2);
		node.style.fontSize = (fontSize * scaleFactor) + "px";
	} catch (e) {}

	try {
		var lineHeight = node.ownerDocument.defaultView.getComputedStyle(node, "").getPropertyValue("line-height");
    		if (lineHeight != "normal") {
			var origLineHeight = node.origLineHeight;
			if (origLineHeight) {
				lineHeight = origLineHeight;
			} else {
				node.origLineHeight = lineHeight;
			}
			lineHeight = lineHeight.substr(0, lineHeight.length - 2);
		        node.style.lineHeight = (lineHeight * scaleFactor) + "px";
    		}
	} catch (e) {}
}

function czGetAllDocuments(frame, documents) {
	var frames = frame.frames;

	documents.push(frame.document);

	for (var i = 0; i < frames.length; i++) {
		czGetAllDocuments(frames[i], documents);
	}

	return documents;
}

function czSetCustomCursorHTML(bSet, cursor) {
 	var contentsRule = "BODY, BODY * { cursor: " + cursor + " !important; }";

	var allDocuments = czGetAllDocuments(window.content, new Array());
	for (var i = 0; i < allDocuments.length; i++) {
		var doc = allDocuments[i];
		if (bSet) {
			var headElements = doc.getElementsByTagName("head");
			var styleElement = doc.createElement("style");
    			styleElement.setAttribute("id", "colozilla-custom-cursor-style-" + cursor);
			styleElement.setAttribute("type", "text/css");
			styleElement.innerHTML = contentsRule;
    			if (headElements.length > 0) {
			        headElements[0].appendChild(styleElement);
    			} else {
			        doc.documentElement.childNodes[0].appendChild(styleElement);
			}
		} else {
			var styleElement = doc.getElementById("colozilla-custom-cursor-style-" + cursor);
			if (styleElement != null) {
        			styleElement.parentNode.removeChild(styleElement);
	    		}
		}
	}
}

function czSetCursorToCrosshair(bSet) {
 	var chromeRule = "window * { cursor: crosshair ! important; }";
	// get the last style sheet
	var sheet = null;
	try {
		sheet = document.styleSheets[document.styleSheets.length - 1];
	} catch (err) {
		dump("Couldn't get document style sheet\n");
	}
	var pos = sheet.cssRules.length;

	// HTML part:
	czSetCustomCursorHTML(bSet, 'crosshair');

	if (bSet) {
		// browser part:
		if (sheet != null) {
			sheet.insertRule(chromeRule, pos);
		}
	} else {
		// browser part:
		if (sheet != null) {
			sheet.deleteRule(pos - 1);
			pos--;
			// make sure that we remove all the rules we added
			// (if we added two identical rules by mistake)
			while (sheet.cssRules[pos - 1].cssText == chromeRule) {
				sheet.deleteRule(pos - 1);
				pos--;
			}
		}
	}
}

function czLaunchDOMInspector() {
	if (gCZSelectedElement != null) {
		inspectDOMNode(gCZSelectedElement);
	} else {
		inspectDOMDocument(window.content.document);
	}
}

function czGetElementLongName(elem) {
	var className = elem.getAttribute("class");
        var idVal = elem.getAttribute("id");

	var name = elem.tagName.toLowerCase();
	if (className) {
		name += "." + className;
	} 

	if (idVal) {
		name += "#" + idVal;
	}

	return name;
}

function czGetCurrentElementLongName() {
	var elem = gCZSelectedElement;
	if (elem == null) {
		return '';
	}

	return czGetElementLongName(elem);
}

function czGetElementTreePath(elem, bReverse) {
	if (!elem) {
		return '';
	}
	var path = czGetElementLongName(elem);
	while (elem.parentNode && elem.parentNode.tagName) {
		elem = elem.parentNode;
        if (!bReverse) {
    		path = czGetElementLongName(elem) + " > " + path;
        } else {
            path = path + " < " + czGetElementLongName(elem);
        }
	}
	return path;
}

function czToggleColorMonitoring(bStopByClick) {
	if (bStopByClick && !gbCZMonitorCurrentColor) {
		return;
	}

	gbCZMonitorCurrentColor = !gbCZMonitorCurrentColor;

	if (!gbCZMonitorCurrentColor) {
		// finished listening

	  	gCZPrevSelectedX = gCZSelectedX;
  		gCZPrevSelectedY = gCZSelectedY;

		// add the selected color to the history palette
		// if the color wasn't sampled from the colorzilla icon
		bClickedOnColorzilla = (gCZSelectedElement != null) && 
					(gCZSelectedElement.getAttribute('id') == 'colorzilla-status-bar-color');
		if ((gCZSelectedColor != null) && !bClickedOnColorzilla) {
			gCZCustomPaletteUI.AddColorToHistory(gCZSelectedColor, czGetCurrentElementLongName());
		}
		czUpdateColorMenuEntries(); 

		// if auto-copy is enable, copy the selected format to the clipboard
		if (gbCZAutoCopyEnabled == 'true') {
			czPerformAutoCopy();
		}

		// clear the outline of the current element if enabled
		czClearCurrentOutline();
	} else {
		// starting listening/show panel
		var wasCollapsed = czShowHideMainPanel(true);
		if (wasCollapsed == 'true') {
			// do not start listening if was hidden:
			gbCZMonitorCurrentColor = false;
			czSetStatusText('');
		} else {
			// check if the Eyedropper mode is supported:
			if (gCZComponent == null) {
				czShowAlert(window, 
							czGetLocaleString('warning'),
							czGetLocaleString('noEyeDropperMessage'));
				gbCZMonitorCurrentColor = false;
			}
		}
	}

	var noCursorElem = document.getElementById('colorzilla-menu-no-crosshair-cursor');
	if (noCursorElem.getAttribute('checked') != 'true') {
		czSetCursorToCrosshair(gbCZMonitorCurrentColor);
		window.setCursor(gbCZMonitorCurrentColor ? 'crosshair' : 'auto');
	}

	// started monitoring
	if (gbCZMonitorCurrentColor) {
		gCZClickListener.startClickListener();
	} else {
		// if stopped by a click, the listeners will be
		// removed by the click handling class
		if (!bStopByClick) {
			gCZClickListener.removeClickListeners();
		}
	}
}

function czKeyPressed(event, bFromKeySet) {
	if (bFromKeySet && gbCtrlSpaceKeyOn) {
		return;
	}

	if (gbCtrlSpaceKeyOn && (!event.ctrlKey || (event.charCode != 32))) {
		return;
	}

	czToggleColorMonitoring();
}

function czRefreshSelectedColor() {
	if ((gCZSelectedX == null) || (gCZSelectedY == null)) {
		return;
	}
	gCZSelectedColor = czGetScreenColor(gCZSelectedX, gCZSelectedY);
	czUpdateColorPanel();
	czUpdateColorMenuEntries();
}

function czClearCurrentOutline() {
	// restore the style of the previous element
	if ((gbCZOutlineCurrentElem == 'true') && gCZSelectedElement) {
		try {
			if (gCZSelectedElementBackupStyle) {
				gCZSelectedElement.setAttribute('style', gCZSelectedElementBackupStyle);
			} else {
				gCZSelectedElement.removeAttribute('style');
			}
		} catch (e) { }
	}
}

function czOnClick(event) {
	if (event.target.hasAttribute('dontclick')) {
		event.target.removeAttribute('dontclick');
		event.stopPropagation();
	      	event.preventDefault();
	}
}

function czOnMouseUp(event) {
	if (gbCZIsPanning) {
		event.stopPropagation();
	      	event.preventDefault();
		gbCZIsPanning = false;
		event.target.setAttribute('dontclick', 'true');
		czSetCustomCursorHTML(false, 'pointer');
		window.setCursor('auto');
	}
}

function czOnMouseDown(event) {
	if (event.target.hasAttribute('dontclick')) {
		event.target.removeAttribute('dontclick');
	}

	if (event.shiftKey && event.altKey) {
		event.stopPropagation();
	      	event.preventDefault();
		gCZPanMouseX = event.screenX;
		gCZPanMouseY = event.screenY;
		gbCZIsPanning = true;
		czSetCustomCursorHTML(true, 'pointer');
		window.setCursor('pointer');
	}
}

function czOnMouseMove(event) {
	if (gbCZIsPanning) {
		event.stopPropagation();
	      	event.preventDefault();
		var dx = event.screenX - gCZPanMouseX;
		var dy = event.screenY - gCZPanMouseY;	
		gCZPanMouseX = event.screenX; 
		gCZPanMouseY = event.screenY;
		event.explicitOriginalTarget.ownerDocument.defaultView.scrollBy(-dx, -dy);
		return;
	} else if (gbCZMonitorCurrentColor) {
		czGetColorUnderCursor(event);
	}
}

function czGetColorUnderCursor(event) {
	
	czClearCurrentOutline();

	gCZSelectedElement = event.target;

	if (gbCZOutlineCurrentElem == 'true') {	
		// outline the element
		gCZSelectedElementBackupStyle = gCZSelectedElement.getAttribute('style');
		if (gCZSelectedElementBackupStyle != null) {
			gCZSelectedElement.setAttribute("style", gCZSelectedElementBackupStyle + "; -moz-outline: 1px dotted red ! important");
		} else {
			gCZSelectedElement.setAttribute("style", "-moz-outline: 1px dotted red ! important");
		}
	}

	gCZSelectedX = event.screenX;
	gCZSelectedY = event.screenY;

	gCZSelectedClientX = event.clientX;
	gCZSelectedClientY = event.clientY;

	czRefreshSelectedColor();
}

function czSwitchColorPickerPaletteSize(event) {
	var newSize = event.target.value;
	czWritePref("palette-size", newSize);
}

function czShowEyedropperHelpDialog() {
	var helpString = czGetLocaleString('press') + " " + 
					gShorcutKeyName + " " + 	
					czGetLocaleString('toStartTheEyedropperMode') + "\n" +
					czGetLocaleString('pressAgainToSampleTheColor') + "\n" +
					czGetLocaleString('theShortcutKeyCanBeChanged');

	czShowAlert(window, czGetLocaleString('eyedropperHelp'), helpString);
}

function czShowColorPicker() {
	if (gbCZMonitorCurrentColor) {
		// turn the color monitoring off:
		czToggleColorMonitoring();
	}
	czUpdateStatusPanel('');

	var bFound = false;

	/*
	/* TBD - do we want the ColorPicker to be a Singleton?
	/*
	// check if the ColorPicker is already open
	const kWindowType = "ColorZilla:ColorPicker";
	var windowMediator = Components.classes["@mozilla.org/appshell/window-mediator;1"]
        	             .getService(Components.interfaces.nsIWindowMediator);
	var windows = windowMediator.getEnumerator(kWindowType);
	var colorPicker;
	while (windows.hasMoreElements()) {
	    colorPicker = windows.getNext().QueryInterface(Components.interfaces.nsIDOMWindowInternal);
	    if (colorPicker.document.documentElement.getAttribute("windowtype") == kWindowType) {
		// found the color picker window:	
		colorPicker.focus();
		bFound = true;
		break;
	    }
	}
	*/

	if (!bFound) {
		// ColorPicker wasn't found, open a new one:
		openDialog('chrome://colorzilla/content/colorzillaColorPickerDialog.xul',
			'',
			'chrome,centerscreen,resizable', 
			gCZSelectedColor, 
			czSetCurrentColor);
	} else {
		// update the ColorPicker with the currently sampled color
		// also, the ColorPicker callback might point to a different window
		// update that too
		// TBD
	}
}

function czColorPanelDoubleClicked(event) {
	// got a double click, do not perform the onclick handler:
	if (gCZEyedropperLaunchTimerID != 0) {
		clearTimeout(gCZEyedropperLaunchTimerID);
		gCZEyedropperLaunchTimerID = 0;
	}

	if (event.button == 0) {
		czShowColorPicker();
	}
}

function czColorPanelClicked(event) {
	if (event.button == 0) {
		// clear any previous timeouts:
		if (gCZEyedropperLaunchTimerID != 0) {
			clearTimeout(gCZEyedropperLaunchTimerID);
			gCZEyedropperLaunchTimerID = 0;
		}
	
		gCZEyedropperLaunchTimerID = setTimeout('czToggleColorMonitoring()', 150);
	}
}


function czSetCurrentColor(color) {
	gCZSelectedColor = color;
	czUpdateColorPanel(true);
	czUpdateColorMenuEntries();
}

function czToggleAutoCopy(event) {
	var menuitem = event.target;

	if (menuitem.getAttribute('checked') == 'true') {
		gbCZAutoCopyEnabled = 'true';
	} else {
		gbCZAutoCopyEnabled = 'false';
	}
	czWritePref("autocopy-enabled", gbCZAutoCopyEnabled);
}

// call this only -after- the menu items have been updated
function czPerformAutoCopy() {
	var valueElemId = null;
	switch (gbCZAutoCopyFormat*1) {
		case COLORZILLA_COPY_RGB: 
			valueElemId = 'colorzilla-menu-copy-rgb';
			break;
		case COLORZILLA_COPY_RGB_PERC: 
			valueElemId = 'colorzilla-menu-copy-rgb-percentage';
			break;

		case COLORZILLA_COPY_HEX: 
			valueElemId = 'colorzilla-menu-copy-rgb-hex';
			break;

		case COLORZILLA_COPY_HEX_SIMPLE: 
			valueElemId = 'colorzilla-menu-copy-rgb-hex-no-number-sign';
			break;
		default: 
			dump("Warning - unknown auto-copy format: " + gbCZAutoCopyFormat + "\n");
	}
	if (valueElemId == null) {
		// not found
		return;
	}	
	
	var valueElem = document.getElementById(valueElemId);
	if (valueElem == null) {
		dump("Warning - didn't find element: " + valueElemId + "\n");
		return;
	}	

	czCopyColorValue(valueElem);
}

function czSelectAutoCopyFormat(format) {
	gbCZAutoCopyFormat = format;
	czWritePref("autocopy-format", format);
}

function czSetupAutoCopy() {
	gbCZAutoCopyEnabled = czReadPref("autocopy-enabled", "false");
	gbCZAutoCopyFormat = czReadPref("autocopy-format", COLORZILLA_COPY_HEX);

	var menuitem = document.getElementById('colorzilla-menu-autocopy-format-' + gbCZAutoCopyFormat);
	menuitem.setAttribute('checked', 'true');
}

function czToggleOutlineCurrentElem(event) {
	var menuitem = event.target;

	if (menuitem.getAttribute('checked') == 'true') {
		gbCZOutlineCurrentElem = 'true';
	} else {
		gbCZOutlineCurrentElem = 'false';
	}

	czWritePref("outline-current-elem", gbCZOutlineCurrentElem);
}

function czToggleShowPageContextMenu(event) {
	var menuitem = event.target;

	var bShow;
	if (menuitem.getAttribute('checked') == 'true') {
		bShow = 'true';
	} else {
		bShow = 'false';
	}

	czWritePref("show-page-context-menu", bShow);
	czShowHidePageContextMenu(bShow == "true");
}

function czShowHidePageContextMenu(bShow) {
	var menu = document.getElementById('colorzilla-pagecontextmenu-mainmenu');
	var sep  = document.getElementById('colorzilla-pagecontextmenu-separator');

	if ((menu != null) && (sep != null)) {
		if (bShow) {
			menu.setAttribute('hidden', 'false');
			sep.setAttribute('hidden', 'false');
		} else {
			menu.setAttribute('hidden', 'true');
			sep.setAttribute('hidden', 'true');
		}
	}
}

function czToggleEnablePanning(event) {
	var menuitem = event.target;
	var bEnabled;
	if (menuitem.getAttribute('checked') == 'true') {
		bEnabled = 'true';
	} else {
		bEnabled = 'false';
	}

	czWritePref("enable-panning", bEnabled);
	czEnableDisablePanning(bEnabled == 'true');
}

function czEnableDisablePanning(bEnable) {
	if (bEnable) {
		addEventListener("click", czOnClick, true);
		addEventListener("mousedown", czOnMouseDown, true);
		addEventListener("mouseup", czOnMouseUp, true);
	} else {
		removeEventListener("click", czOnClick, true);
		removeEventListener("mousedown", czOnMouseDown, true);
		removeEventListener("mouseup", czOnMouseUp, true);
	}
}

function czUpdateLowercaseHexa(elem) {
	gbCZLowerCaseHexa = elem.getAttribute('checked');
	czWritePref("lowercase-hexa", gbCZLowerCaseHexa);
	// convert to real boolean
	gbCZLowerCaseHexa = (gbCZLowerCaseHexa == 'true');
}

function czSetupGUI() {
	var paletteSize = czReadPref("palette-size", 16);
	var menuElem = document.getElementById("colorzilla-color-picker-palette-" + paletteSize);
	if (menuElem != null) {
		menuElem.setAttribute("checked", true);
	}

 	// check DOM Inspector existance:
	if (typeof inspectDOMNode == "undefined") {
		document.getElementById('colorzilla-menu-dom-inspector').setAttribute('hidden', 'true');
		document.getElementById('colorzilla-menu-dom-inspector-separator').setAttribute('hidden', 'true');

		document.getElementById('colorzilla-menu-dom-inspector-context').setAttribute('hidden', 'true');
		document.getElementById('colorzilla-menu-dom-inspector-separator-context').setAttribute('hidden', 'true');
	}

	// setup the status bar formatting string
	gCZStatusBarFormat = czReadPref("status-bar-format", "%rgb% | %hex% | %dist% | %element%");

	czSetupAutoCopy();

	gbCZOutlineCurrentElem = czReadPref("outline-current-elem", "true");
	var menuItem = document.getElementById('colorzilla-menu-outline-current-node');
	menuItem.setAttribute('checked',  gbCZOutlineCurrentElem);

	// setup the main page context menu:
	var bShowContext = czReadPref("show-page-context-menu", "true");
	if (bShowContext == "false") {
		czShowHidePageContextMenu(false);
	}
	menuItem = document.getElementById('colorzilla-menu-show-page-context-menu');
	menuItem.setAttribute('checked',  bShowContext);

	// setup panning if needed:
	var bPanningOn = czReadPref("enable-panning", 'true');
	if (bPanningOn == 'true') {
		czEnableDisablePanning(true);
	}
	menuItem = document.getElementById('colorzilla-menu-enable-panning');
	menuItem.setAttribute('checked',  bPanningOn);

	// setup the lowercase hexa option:
	gbCZLowerCaseHexa = czReadPref("lowercase-hexa", "false");
	menuItem = document.getElementById('colorzilla-menu-lowercase-hexa');
	menuItem.setAttribute('checked',  gbCZLowerCaseHexa);
	czUpdateLowercaseHexa(menuItem);
}

function czChangeZoom(event) {
	var zoom = event.target.getAttribute('value');
	var zoomFactor = zoom / 100;
	czZoomElements(window.content.document, zoomFactor); 
}

function czBuildZoomMenu() {
	const zoomFactors  = [20, 30, 50, 70, 90, 100, 110, 120, 150, 180, 200, 
				250, 300, 400, 500, 600, 700, 800, 900, 1000];

	var zoomMenuIds = [ 'colorzilla-menu-zoom', 
						'colorzilla-menu-zoom-context' ];
	for (iZoomMenu = 0; iZoomMenu < zoomMenuIds.length; iZoomMenu++) {
   		var zoomMenu = document.getElementById(zoomMenuIds[iZoomMenu]);
		for (var i=0; i < zoomFactors.length; i++) {
			var zoomFactor = zoomFactors[i];
			newElement = document.createElement('menuitem');
       	 		newElement.setAttribute('label', zoomFactor + "%");
				newElement.setAttribute('tooltiptext', zoomFactor + "%");
        		newElement.setAttribute('value', zoomFactor);
        		newElement.setAttribute('oncommand', 'czChangeZoom(event)');
        		zoomMenu.appendChild(newElement);
		}
	}
}


function czSetShortcuts() {
	var keyElem = document.getElementById('key_colorzilla-main');
	gbCtrlSpaceKeyOn = false;

	var modifier = czReadPref("shortcut-modifier", '');
	var key = czReadPref("shortcut-key", '');
	keyElem.setAttribute('disabled', 'false');

	if ((modifier != '') && (key != '')) {
	    	keyElem.removeAttribute('key');
	        keyElem.removeAttribute('keycode');
        	keyElem.removeAttribute('modifiers');
	   	if ((modifier == 'control') && (key == 'VK_SPACE')) {
			gbCtrlSpaceKeyOn = true;
			keyElem.setAttribute('disabled', 'true');
		} else {
			if (key.length == 1) {
				keyElem.setAttribute('key', key);
			} else {
				keyElem.setAttribute('keycode', key);
			}
			keyElem.setAttribute('modifiers', modifier.replace("+", ","));
		}
	} else {
		// default names (only for display):
		modifier = "Shift";
		key = "Escape";
  	}

	if (gbCtrlSpaceKeyOn) {
		addEventListener("keypress",  czKeyPressed, false);
		gShorcutKeyName = "Ctrl+Space";
        } else {
		removeEventListener("keypress",  czKeyPressed, false);
		gShorcutKeyName = modifier + "+" + key;
	}
}

function czInitOverlay() {
	iosartInstallPlatformLibraryIfNeeded('ColorZilla',
				'{6AC85730-7D0F-4de0-B3FA-21142DD85326}');

	czInitComponent();

	czSetupGUI();

	addEventListener("mousemove", czOnMouseMove, false);

	czSetShortcuts();

	czBuildZoomMenu();

	gCZIOUtils.InstallDefaultPalettesIfNeeded();
}

addEventListener("load", czInitOverlay, false);


// abbrev. CZ == ColorZilla

// the native ColorZilla component
var gCZComponent = null;

// are we looking at the cuttent color?
var gbCZMonitorCurrentColor = false;

// the last selected color
var gCZSelectedColor = null;

// the last statusbar message
var gCZLastStatusValue = null;


// last selected coordinates and element
var gCZSelectedX = null;
var gCZSelectedY = null;
var gCZSelectedElement = null;

// last selected coordinated relative to the browser screen
var gCZSelectedClientX = null;
var gCZSelectedClientY = null;

// element highlighting
var gCZSelectedElementBackupStyle = null;
var gbCZOutlineCurrentElem;

// previouly selected coordinates and element
var gCZPrevSelectedX = null;
var gCZPrevSelectedY = null;

// is the ctrl+space shortcut key on
var gbCtrlSpaceKeyOn = false;

// var the name of the current shorcut key:
var gShorcutKeyName;

// saved status text for restoring later:
var gCZSavedStatus = '';

// timer ID for launching the EyeDropper:
var gCZEyedropperLaunchTimerID = 0;

// the status bar formatting string
var gCZStatusBarFormat = '';

// is the auto-copy enabled
var gbCZAutoCopyEnabled = false;
// auto-copy format
var gbCZAutoCopyFormat = '';

// panning mode
var gbCZIsPanning = false;
var gCZPanMouseX = -1;
var gCZPanMouseY = -1;
