/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is ColorZilla Mozilla Extension.
 * The Initial Developer of the Original Code is Alex Sirota <alex@elbrus.com>. 
 * Portions created by Alex Sirota are Copyright (C) 2004 Alex Sirota. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

const CZ_PALETTE_LIST_CACHE_FILE  = "palettes.xml";

const CZ_PALETTE_SORT_BY_INDEX	= 0;
const CZ_PALETTE_SORT_BY_VALUE 	= 1;
const CZ_PALETTE_SORT_BY_NAME 	= 2;

const CZ_PALETTE_SORT_ASCENDING		= 0;
const CZ_PALETTE_SORT_DESCENDING	= 1;

function CZIOUtils() {
	this.dirService = Components.classes["@mozilla.org/file/directory_service;1"].
				getService(Components.interfaces.nsIProperties);
}

CZIOUtils.prototype = {
	ReadFile : function(file) { 
	        try {
        	   var inputStream =  Components.classes["@mozilla.org/network/file-input-stream;1"].
					createInstance(Components.interfaces.nsIFileInputStream);
	           inputStream.init(file,0x01,0444,null);
 
        	   var scriptableIO = Components.classes["@mozilla.org/scriptableinputstream;1"].
					createInstance(Components.interfaces.nsIScriptableInputStream);
	           scriptableIO.init(inputStream);
	        } catch (e) { dump("Error opening file: " + e + "\n"); }
		var str = null;
        	try {
                	str = scriptableIO.read(file.fileSize);
	        } catch (e) { dump("Error reading from file: " + e + "\n"); }

        	scriptableIO.close();
	        inputStream.close();
        	return str;
	},

	WriteFileImpl: function(file, contents, mode) {
		if (!file.exists()) {
			try {
				file.create(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 0664);
			} catch(e) { dump("Couldn't create file: " + e + "\n"); }
		}

		if (!file.isWritable()) {
			file.permissions = 0644;
		}

		try {
			var outputStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
						.createInstance( Components.interfaces.nsIFileOutputStream );
			outputStream.init(file, mode, 0664, 0);
			var result = outputStream.write(contents, contents.length);
		} catch(e) { dump("Couldn't write to file: " + e + "\n"); }
		outputStream.close();
	},

	WriteFile: function(file, contents) {
		return this.WriteFileImpl(file, contents, 0x04 | 0x08 | 0x20);
	},

	AppendToFile: function(file, contents) {
		return this.WriteFileImpl(file, contents, 0x04 | 0x10);
	},

	GetPalettesFolder: function() {
		var paletteDir = this.dirService.get("ProfD", Components.interfaces.nsIFile);
		paletteDir.append("colorzilla");	
		paletteDir.append("palettes");	
		return paletteDir;
	},

	InstallDefaultPalettesIfNeeded: function() {
		var target = this.dirService.get("ProfD", Components.interfaces.nsIFile);
		var source = target.clone();

		target.append("colorzilla");
		if (!target.exists()) {
			target.create(target.DIRECTORY_TYPE, 0755);
		}

		target.append("palettes");
		if (!target.exists()) {
			target.create(target.DIRECTORY_TYPE, 0755);
		}

		if (iosartIsInFirefox()) {
			// in Firefox the default palettes are located in the extension folder
	    		source.append("extensions");
			source.append("{6AC85730-7D0F-4de0-B3FA-21142DD85326}");
			source.append("defaults");
			source.append("palettes");
		} else {
			// in Mozilla, install.js installed the default palettes into 
			// Profile/colorzilla/palettes/default
	    		source.append("colorzilla");
	    		source.append("palettes");
	    		source.append("default");
		}

		var markerFile = source.clone();
		markerFile.append("install.marker.txt");
		if (!markerFile.exists()) {
			// the marker file has been removed, meaning we already installed the palettes
			return;
		}

		// remove the marker file to indicate that we are installing palettes
		markerFile.remove(false);

		var files = source.directoryEntries;
		while (files.hasMoreElements()) {
			var file = files.getNext().QueryInterface(Components.interfaces.nsILocalFile);
			if (!file.isFile()) {
				continue;
			}


			var targetFile = target.clone();
			targetFile.append(file.leafName);
			if (targetFile.exists()) {
				var backupDir = target.clone();
				backupDir.append("backup"); 
				if (!backupDir.exists()) {
					backupDir.create(backupDir.DIRECTORY_TYPE, 0755);
				}	
				dump(file.leafName + " exists, backing it up\n");
				targetFile.moveTo(backupDir, '');
			}

			dump("Installing " + file.leafName + "\n");
			file.copyTo(target, '');
		}

	}
}

function CZColorEntry(value, name, index) {
	this.value = value;
	this.name = name;
	this.indexInPalette = index;
}

// static methods
CZColorEntry.CompareByNameAscending = function(one, two) {
	var oneName = one.name.toLowerCase();
	var twoName = two.name.toLowerCase();
	return czCompareTwoStrings(oneName, twoName);
}

CZColorEntry.CompareByNameDescending = function(one, two) {
	return CZColorEntry.CompareByNameAscending(two, one);
}

CZColorEntry.CompareByValueAscending = function(one, two) {
	var col = czRGBHexaAttributeToCol(one.value);
	var r = czGetRValue(col);
	var g = czGetGValue(col);
	var b = czGetBValue(col);
	var grayScaleA = czRGBToGrayscale(r, g, b);
	
	col = czRGBHexaAttributeToCol(two.value);
	r = czGetRValue(col);
	g = czGetGValue(col);
	b = czGetBValue(col);
	grayScaleB = czRGBToGrayscale(r, g, b);

	return grayScaleA - grayScaleB;
}

CZColorEntry.CompareByValueDescending = function(one, two) {
	return CZColorEntry.CompareByValueAscending(two, one);
}

CZColorEntry.CompareByIndexAscending = function(one, two) {
	return one.indexInPalette - two.indexInPalette;
}

CZColorEntry.CompareByIndexDescending = function(one, two) {
	return two.indexInPalette - one.indexInPalette;
}

CZColorEntry.CompareByHueAscending = function(one, two) {
	var col = czRGBHexaAttributeToCol(one.value);
	var r = czGetRValue(col);
	var g = czGetGValue(col);
	var b = czGetBValue(col);
	var hsvA = czRGBToHSV(r, g, b);
	
	col = czRGBHexaAttributeToCol(two.value);
	r = czGetRValue(col);
	g = czGetGValue(col);
	b = czGetBValue(col);
	var hsvB = czRGBToHSV(r, g, b);
	
	if (hsvA.h != hsvB.h) {
		return hsvA.h - hsvB.h;
	} else if (hsvA.s != hsvB.s) {
		return hsvA.s - hsvB.s;
	} else {
		return hsvA.v - hsvB.v;
	}
}

CZColorEntry.CompareByHueDescending = function(one, two) {
	return CZColorEntry.CompareByHueAscending(two, one);
}

function CZColorPalette() {
	this.name = null;
	this.colors = null;
	this.bEditable = false;
	// was this palette resorted
	this.bReSorted = false;
	this.nColumns = 16;
}

CZColorPalette.prototype = {
	Clone : function() {
		var newPalette = new CZColorPalette();
		newPalette.name = this.name;
		newPalette.bEditable = this.bEditable;
		newPalette.bReSorted = this.bReSorted;
		newPalette.colors = new Array(this.colors.length);
		newPalette.nColumns = this.nColumns;
		for (var i=0; i < this.colors.length; i++) {
			newPalette.colors[i] = this.colors[i];
		}
		return newPalette;
	},

	TrimString : function(string) {
		// collapse multiple white space characters into a single one:
		string = string.replace(/\s+/g, " ");

		// remove the leading space
		if (string.charAt(0) == ' ') {
			string = string.substr(1);
		}
		
		// remove the trailing space
		if (string.charAt(string.length - 1) == ' ') {
			string = string.substr(0, string.length - 1);
		}

		return string;
	}, 

	//
	// bParseHeaderOnly - 	optional. if true, only the header will be parsed
	//			and then the parsing will stop
	// 
	FromGIMPFile : function(file, bParseHeaderOnly) {
		if (bParseHeaderOnly === undefined) {
			// default value
			bParseHeaderOnly = false; 
		}

		if (!file.exists()) {	
			return null;
		}
		var str = gCZIOUtils.ReadFile(file);
		// error reading file
		if (str == null) {
			return;
		}

		this.colors = new Array();
		var lines = str.split(/[\n\r]/);
		var bInHeader = true;
		// meta palette includes CSS color names (like ActiveBorder) instead of colors
		var bMetaPalette = false;
		// the helper element will help caclulate the real color values in the meta palette
		var metaHelperElem;
		
		for (var i = 0; i < lines.length; i++) {
			var line = lines[i];

			// check if this is an editable palette (# CZEditable appears)
			if (bInHeader && line.match(/CZEditable/i)) {
				this.bEditable = true;
				continue;
			}

			if (bInHeader && line.match(/CZMetaPalette/i)) {
				bMetaPalette = true;
				metaHelperElem = document.createElement('box');
				metaHelperElem.setAttribute('id', 'colorzilla-metapalette-helper');
				continue;
			}

			// meta palette entry line typically looks like this:
			// # CZ Value Description
			if (bMetaPalette && line.match(/#\s*CZ\s+([^\s]+)\s+(.*)/)) {
				var value = RegExp.$1;
				metaHelperElem.setAttribute('style', 'color: ' + value);
				var realColor = metaHelperElem.ownerDocument.defaultView.
							getComputedStyle(metaHelperElem, "").getPropertyValue("color");

				var colorRef = czRGBAttributeToCol(realColor);
				var colorValueHex = czColToRGBHexaAttribute(colorRef);
				var name = RegExp.$1 + " - " + RegExp.$2;
				name = this.TrimString(name);
				var color = new CZColorEntry(colorValueHex, name, this.colors.length)
				this.colors.push(color);
				continue;
			}


			// ignore empty lines and other (non-colozilla) comments
			if ((line.length == 0) || (line.charAt(0) == '#')) {
				continue;
			}
		
			// check if this is a name line: 
			if (bInHeader && line.match(/^name:\s*(.*)/i)) {
				this.name = this.TrimString(RegExp.$1);
				continue;
			}

			// check if this is number of columns line
			if (bInHeader && line.match(/^columns:\s*(.*)/i)) {
				this.nColumns = this.TrimString(RegExp.$1);
				continue;
			}

			// check if this is a color line:
			line = this.TrimString(line);
			if (line.match(/^(\d+)\s+(\d+)\s+(\d+)\s*(.*)/)) {
				// found a color, no longer in the header
				bInHeader = false;

				if (bParseHeaderOnly) {
					// we were asked to parse only the header. stop the parsing
					break;
				}

				var colorValueHex = czColToRGBHexaAttribute(czRGBToColor(RegExp.$1, RegExp.$2, RegExp.$3));
				var name = RegExp.$4;
				name = this.TrimString(name);
				if (name.toLowerCase() == 'untitled') {
					name = null;
				}
				var color = new CZColorEntry(colorValueHex, name, this.colors.length)
				this.colors.push(color);
			}
		}

		// if no name found, the use the filename
		if ((this.name == null) || (this.name == '')) {
			this.name = file.leafName;
		}

		return this.colors;
	},

	ToGIMPFile : function(file) {
		if (file.exists()) {	
			// remove the file if it exists
			file.remove(false);
		}

		var paletteStr = ''; 
		paletteStr += "GIMP Palette\n";
		paletteStr += "Name: " + this.name + "\n";
		paletteStr += "Columns: " + this.nColumns + "\n";
		paletteStr += "#\n";
		paletteStr += "# ColorZilla " + this.name + " palette\n";
		paletteStr += "# CZEditable\n";
		paletteStr += "#\n";
		for (var i = 0; i < this.colors.length; i++) {
			var color = this.colors[i];
			var colorRef = czRGBHexaAttributeToCol(color.value);
			var r = czGetRValue(colorRef);
			var g = czGetGValue(colorRef);
			var b = czGetBValue(colorRef);
			var name = color.name;
			if (color.name == null) {
				name = 'untitled';
			}
			paletteStr += r + " " + g + " " + b + "\t" + name + "\n";	
		}
		
		gCZIOUtils.WriteFile(file, paletteStr);
	},

	ToCSSFile : function(file) {
		if (file.exists()) {	
			// remove the file if it exists
			file.remove(false);
		}

		var paletteStr = ''; 
		paletteStr += "/*\n";
		paletteStr += "   ColorZilla exported palette (" + this.name + ")\n";
		paletteStr += "*/\n\n";
		for (var i = 0; i < this.colors.length; i++) {
			var color = this.colors[i];
			var colorRef = czRGBHexaAttributeToCol(color.value);
            var rgb = czColToRGBAttribute(colorRef)
			var name = color.name;
			if (color.name != null) {
			    paletteStr += "/* " + color.name + " */\n";
			}
			paletteStr += color.value + "\n";
			paletteStr += rgb + "\n";
			paletteStr += "\n";
		}
		
		gCZIOUtils.WriteFile(file, paletteStr);
	},


	FromFile : function(file, bParseHeaderOnly) {
		return this.FromGIMPFile(file, bParseHeaderOnly);
	},

	ToFile : function(file) {
		return this.ToGIMPFile(file);
	},

	ReSortByHue : function(file) {
		this.FromFile(file);
		this.colors.sort(CZColorEntry.CompareByHueAscending);
		this.ToFile(file);
	}
}

function CZPaletteList() {
	this.paletteData = null;
	this.DOMParser	= Components.classes["@mozilla.org/xmlextras/domparser;1"].
				createInstance(Components.interfaces.nsIDOMParser);
  	this.serializer = Components.classes["@mozilla.org/xmlextras/xmlserializer;1"].
				createInstance(Components.interfaces.nsIDOMSerializer);
}

CZPaletteList.prototype = {
	FromFile : function(file) {
		var xmlText = gCZIOUtils.ReadFile(file);
		if (!xmlText) {
			return false;
		}
		this.paletteData = this.DOMParser.parseFromString(xmlText, "text/xml");
		return true;
	},

	ToFile : function(file) {
	    	gCZIOUtils.WriteFile(file, this.serializer.serializeToString(this.paletteData));
	},

	// calculate the checksum of the list of files in the palettes folder
	// used to determine when the list changes
	CalcCheckSum : function(dir) {
		var sum = 0;
		var files = dir.directoryEntries;
		var fileList = '';
		while (files.hasMoreElements()) {
			var file = files.getNext().QueryInterface(Components.interfaces.nsILocalFile);
			if (!file.isFile() || (file.leafName == CZ_PALETTE_LIST_CACHE_FILE)) {
				continue;
			}
			fileList += file.leafName;
		}

		for (var i=0; i < fileList.length; i++) {
			sum += fileList.charCodeAt(i);
		}
		return sum;
	},

	GetListCheckSum : function() {
		if (!this.paletteData) {
			return -1;
		}

		var checkSumNode = this.paletteData.getElementsByTagName("checksum")[0];		
		return checkSumNode.getAttribute('value');
	},

	Init : function(dir) {
		if (!dir.exists() || !dir.isDirectory()) {	
			return null;
		}

		var cacheFile = dir.clone();
		cacheFile.append(CZ_PALETTE_LIST_CACHE_FILE);
	
		var checkSum = this.CalcCheckSum(dir);
	
		// check if the cache file exists, if so load from cache
		if (cacheFile.exists()) {
			if (this.FromFile(cacheFile)) {
				// verify the checksum (folder hasn't changed)
				var savedCheckSum = this.GetListCheckSum();
				if (savedCheckSum == checkSum) {
					dump("Palette list cache hit\n");
					return;
				}
				dump("Palette list cache miss - folder changed\n");
			}
		}

		// else, parse all the files in the folder

		// create a new XML DOM:
		this.paletteData = document.implementation.createDocument("","",null);
		var rootNode = this.paletteData.createElement("palette-cache");

		var palettesNode = this.paletteData.createElement("palettes");
		
		var files = dir.directoryEntries;
		var paletteNodesArray = new Array();
		while (files.hasMoreElements()) {
			var file = files.getNext().QueryInterface(Components.interfaces.nsILocalFile);
			if (!file.isFile() || (file.leafName == CZ_PALETTE_LIST_CACHE_FILE)) {
				continue;
			}

			var palette = new CZColorPalette();

			// set the second parameter to true, to indicate that we only want to parse the header
			palette.FromFile(file, true);

			var paletteNode = this.paletteData.createElement("palette");
			paletteNode.setAttribute("name", palette.name); 
			paletteNode.setAttribute("filename", file.path); 
			// we are parsing headers only, this is no longer available:
			// paletteNode.setAttribute("ncolors", palette.colors.length); 

			paletteNodesArray.push(paletteNode);
		}

		// sort the palettes by name
		var sortFunc = function (a,b) { return czCompareTwoStrings(a.getAttribute('name'), b.getAttribute('name')); }
		paletteNodesArray.sort(sortFunc);

		// add the sorted palettes to the DOM
		for (var i=0; i < paletteNodesArray.length; i++) {
			palettesNode.appendChild(paletteNodesArray[i]);
		}

		// add the current list checksum to the data
		var checkSumNode = this.paletteData.createElement("checksum");
		checkSumNode.setAttribute("value", checkSum);
		rootNode.appendChild(checkSumNode);

		// add the palette list
		rootNode.appendChild(palettesNode);

		// add the root node
		this.paletteData.appendChild(rootNode);

		// save the palette list in the cache file
		this.ToFile(cacheFile);
	}
}

function CZColorPaletteUI() {
	this.kPaletteListElemId = 'colorzilla-color-palettes-list';
	this.kPaletteViewModeListId = 'colorzilla-color-palettes-view-mode-list';
	this.kPaletteListBoxID = 'colorzilla-palette-color-list';

	this.previouslyMarkedColorTile = null;
	this.prevHistoryColor = null;
	this.prevHistoryColorName = null;

	this.currentPaletteMetaData = null;
	this.currentPalette = null;
	this.currentColorIndex = -1;

	this.sortBy = czReadPref("colors-sort-by", CZ_PALETTE_SORT_BY_INDEX);
	this.sortBy *= 1; // make sure it's a number
	this.sortDir = czReadPref("colors-sort-dir", CZ_PALETTE_SORT_ASCENDING);
	this.sortDir *= 1; // make sure it's a number
}

CZColorPaletteUI.prototype = {
	Init : function() {
		var paletteDir = gCZIOUtils.GetPalettesFolder();
		var paletteList = new CZPaletteList();
		paletteList.Init(paletteDir);
		this.PopulatePaletteList(paletteList);
		this.SelectInitialPaletteInList();
		this.SelectInitialPaletteView();
	}, 

	IsPaletteEditable : function(palette) {
		return (palette.bEditable);
	},

	ChangePaletteSorting : function(event, sortBy) {
		if (this.sortBy != sortBy) {
			// change the sort column
			this.sortBy = sortBy;
		} else {
			// change the sort dir
			this.sortDir *= 1; // make sure its a number
			this.sortDir = (this.sortDir == CZ_PALETTE_SORT_ASCENDING ? 
						CZ_PALETTE_SORT_DESCENDING :
						CZ_PALETTE_SORT_ASCENDING);
		}

		czWritePref("colors-sort-by", this.sortBy );
		czWritePref("colors-sort-dir", this.sortDir );

		this.FillList(this.currentPalette);
	},	

	SetSortMarkersIfNeeded : function(listBox) {
		var headers = listBox.getElementsByTagName('listheader');
		var headerColor = headers[0];
		var headerValue = headers[1];
		var headerName = headers[2];

		// first, remove the existing markers
		headerColor.removeAttribute('sortDirection');
		headerValue.removeAttribute('sortDirection');
		headerName.removeAttribute('sortDirection');

		// if we are in the defaul sorting, return
		if ((this.sortBy == CZ_PALETTE_SORT_BY_INDEX) &&
			(this.sortDir == CZ_PALETTE_SORT_ASCENDING)) {
			return;
		}
	
		var target = null;
		switch (this.sortBy) {
			case CZ_PALETTE_SORT_BY_NAME: 
				target = headerName;
				break;

			case CZ_PALETTE_SORT_BY_VALUE: 
				target = headerValue;
				break;

			case CZ_PALETTE_SORT_BY_INDEX: 
				target = headerColor;
				break;
			default:
				dump("Warning: unknown sort order: " + this.sortBy + "\n");
		}
			
		if (target == null) {
			return;
		}

		var sortDirValue = (this.sortDir == CZ_PALETTE_SORT_ASCENDING ?  'ascending' : 'descending');
		target.setAttribute('sortDirection', sortDirValue);
	},

	SortPaletteIfNeeded : function(palette) {
		if ((this.sortBy == CZ_PALETTE_SORT_BY_INDEX) &&
			(this.sortDir == CZ_PALETTE_SORT_ASCENDING) && 
			(!palette.bReSorted)) {
			// this is the default sorting and the palette hasn't been 
			// sorted yet - do nothing
			return palette;
		}

		palette = palette.Clone();
		var compareFunc = null;
		switch (this.sortBy) {
			case CZ_PALETTE_SORT_BY_NAME: 
				compareFunc = this.sortDir == CZ_PALETTE_SORT_ASCENDING ? 
						CZColorEntry.CompareByNameAscending : 
						CZColorEntry.CompareByNameDescending;
				break;

			case CZ_PALETTE_SORT_BY_VALUE: 
				compareFunc = this.sortDir == CZ_PALETTE_SORT_ASCENDING ? 
						CZColorEntry.CompareByValueAscending : 
						CZColorEntry.CompareByValueDescending;
				break;

			case CZ_PALETTE_SORT_BY_INDEX: 
				compareFunc = this.sortDir == CZ_PALETTE_SORT_ASCENDING ? 
						CZColorEntry.CompareByIndexAscending : 
						CZColorEntry.CompareByIndexDescending;
				break;
			default:
				dump("Warning: unknown sort order: " + sortBy + "\n");
		}
		
		if (compareFunc == null) {
			return;
		}

		palette.colors.sort(compareFunc);
		palette.bReSorted = true;
		return palette;
	},

	FillList : function(palette) {
		var listItem;
		var listCell;
		const kPaletteListBoxContainerID = 'colorzilla-palette-color-list-container';

		var listBoxContainer = document.getElementById(kPaletteListBoxContainerID);
	
		// clone the template to get a fresh list box	
		listBox = listBoxContainer.firstChild.cloneNode(true);
		// this should always be visible (the container - father - may be hidden):
		listBox.setAttribute("hidden", "false"); 
		listBox.setAttribute("onselect", "gCZCustomPaletteUI.CustomColorClicked(event)"); 
		if (this.IsPaletteEditable(palette)) {
			listBox.setAttribute("context", 'colorzilla-edit-palette-popup');
		}
	
		palette = this.SortPaletteIfNeeded(palette);	

		for (var i = 0; i < palette.colors.length; i++) {
			listItem = document.createElement('listitem');
			if (palette.colors[i].name) {
				//bug, tooltip appears at the top left part of the screen
				//listItem.setAttribute("tooltiptext", palette.colors[i].name); 
			}
			listItem.value = palette.colors[i]; 

			listCell = document.createElement('listcell');
			listCell.setAttribute("class", "color-patch");
			listCell.setAttribute("style", "background-color: " + palette.colors[i].value); 
			listItem.appendChild(listCell);


			listCell = document.createElement('listcell');
			// listCell.setAttribute("class", "colorzilla-monospace");
			listCell.setAttribute("label", palette.colors[i].value); 
			listItem.appendChild(listCell);

			listCell = document.createElement('listcell');
			listCell.setAttribute("label", palette.colors[i].name); 
			listItem.appendChild(listCell);

			listBox.appendChild(listItem);
		}	

		// remove the old list box, if exists:
		var oldListBox = document.getElementById(this.kPaletteListBoxID);
		if (oldListBox) {
			listBoxContainer.removeChild(oldListBox);
		}

		this.SetSortMarkersIfNeeded(listBox);

		// add this list box
		listBox.setAttribute("id", this.kPaletteListBoxID); 
		listBoxContainer.appendChild(listBox);

	},

	FillCustomPalette : function (palette) {
		const kCustomPaletteContainerID = 'colorzilla-palette-custom-palette-container';
		const kCustomPaletteID = 'colorzilla-palette-custom-palette';


		var paletteContainer = document.getElementById(kCustomPaletteContainerID);

		var row = null;
		var spacer;
		var nItemsInRow = palette.nColumns;

		//var tileWidth = Math.round(80 / nItemsInRow);
		var tileWidth = Math.round(200 / nItemsInRow);

		var bFirstRow = true;
		var paletteElem = document.createElement('vbox');
		if (this.IsPaletteEditable(palette)) {
			paletteElem.setAttribute("context", 'colorzilla-edit-palette-popup');
		}
		for (var i = 0; i < palette.colors.length; i++) {

			// create a new row and add the previous one
			if ((i % nItemsInRow) == 0) {
				if (row != null) {
					paletteElem.appendChild(row);
					bFirstRow = false;
					row = null;
				}
				row = document.createElement('hbox');
				//row.setAttribute("flex", "1");
			}

			spacer = document.createElement('spacer');

			var tileClass = "colorzilla-color-tile";
			if (bFirstRow) {
				tileClass += " colorzilla-color-tile-top";
			}
			// first tile in the row
			if ((i % nItemsInRow) == 0) {
				tileClass += " colorzilla-color-tile-leftmost";
			}

		
			//spacer.setAttribute("flex", "1");
			spacer.setAttribute("class", tileClass); 
			spacer.value = palette.colors[i]; 
			spacer.setAttribute("style", "background-color: " + palette.colors[i].value + ";" +
						     "width: " + tileWidth + "px;");
			var name = palette.colors[i].value;
			if (palette.colors[i].name != null) {
				name = palette.colors[i].name + " (" + name + ")";
			}
			spacer.setAttribute("tooltiptext", name); 
			spacer.setAttribute("onclick", "gCZCustomPaletteUI.CustomColorClicked(event)"); 

			row.appendChild(spacer);
		}
	
		// append the last row if needed:
		if (row != null) {
			paletteElem.appendChild(row);
			row = null;
		}

		// remove the old palette, if exists:
		var oldPalette = document.getElementById(kCustomPaletteID);
		if (oldPalette) {
			paletteContainer.removeChild(oldPalette);
		}

		// append the palette into the container
		paletteElem.setAttribute("id", kCustomPaletteID); 
		paletteElem.style.width = (tileWidth*nItemsInRow + 40) + "px";
		paletteContainer.appendChild(paletteElem);
	},

	PaletteIsSpecial : function(paletteName) {
		if (paletteName == czGetLocaleString('history')) {
			return true;
		}
		if (paletteName == czGetLocaleString('favorites')) {
			return true;
		}
		return false;
	},

	PopulatePaletteList : function(paletteList) {

		var listElem = document.getElementById(this.kPaletteListElemId);

		if (paletteList.paletteData == null) {
			// no data, return
			var menuItem = document.createElement('menuitem');
			menuItem.setAttribute('label', "--------------");
			listElem.firstChild.appendChild(menuItem);
			return;
		}


		var palettes = paletteList.paletteData.getElementsByTagName("palette");
		for (var i = 0; i < palettes.length; i++) {
			var palette = palettes[i];
			var name = palette.getAttribute('name');
			var menuItem = document.createElement('menuitem');
			menuItem.setAttribute('label', name);

			var bSpecial = this.PaletteIsSpecial(name);
			if (bSpecial) {
				menuItem.setAttribute('special', "true");
			}

			// we are parsing headers only, this is no longer available:
			// var nColors = palette.getAttribute('ncolors');
			menuItem.setAttribute('tooltiptext', name);

			menuItem.value = palette;
			menuItem.setAttribute('oncommand', "gCZCustomPaletteUI.SwitchToPalette(event)");
			listElem.firstChild.appendChild(menuItem);
		}
	}, 

	SelectInitialPaletteInList : function() {
		// the default value is '-1', should be found in the palette list
		var paletteName = czReadPref("last-palette", 'Hues and Saturations');	

		var listElem = document.getElementById(this.kPaletteListElemId);
		var children = listElem.getElementsByTagName("menuitem");

		if (children.length == 0) {
			// no palettes, nothing to do
			return;
		}

		var foundPalette = null;
		for (var i=0; i < children.length; i++) {
			var child = children[i];
			var name = child.value.getAttribute('name');
			if (name == paletteName) {
				listElem.selectedIndex = i;
				foundPalette = child.value;
				break;
			}
		}

		if (foundPalette == null) {
			// by default, select the first item
			listElem.selectedIndex = 0;
			foundPalette = children[0].value;
		}

		this.SwitchToPaletteImpl(foundPalette);
	},

	SelectInitialPaletteView : function() {
		var viewMode = czReadPref("last-palette-view-mode", 'thumbnails');

		var modeListElem = document.getElementById(this.kPaletteViewModeListId);
		var children = modeListElem.getElementsByTagName("menuitem");
	
		var bFound = false;	
		for (var i=0; i < children.length; i++) {
			var child = children[i];
			if (child.value == viewMode) {
				modeListElem.selectedIndex = i;
				bFound = true;
				break;
			}
		}

		if (!bFound) {
			dump("Warning - couldn't find view mode " + viewMode + "\n");
			return;
		}

		this.CustomPaletteChangeViewImpl(viewMode);
	},

	ClearPaletteMarkers : function() {
		// clear the listbox selection
		var paletteAsListElem = document.getElementById(this.kPaletteListBoxID);
		paletteAsListElem.selectedIndex = -1;

		// clear the palette selection
		if (this.previouslyMarkedColorTile != null) {
			this.previouslyMarkedColorTile.removeAttribute("current");
			this.previouslyMarkedColorTile = null;
		}
		this.currentColorIndex = -1;
	},

	AddColorToPalette : function(col, name, paletteFilename, paletteName) {
		var paletteFile = gCZIOUtils.GetPalettesFolder();
		paletteFile.append(paletteFilename);
		if (!paletteFile.exists()) {
			var header = '';
			header = "GIMP Palette\n" + 
					"Name: " + paletteName + "\n" +
					"#\n" +
					"# ColorZilla " + paletteName + " Palette\n" +
					"# CZEditable\n" + 
					"#\n";
					
			gCZIOUtils.WriteFile(paletteFile, header);
		}

		var r = czGetRValue(col);
		var g = czGetGValue(col);
		var b = czGetBValue(col);
		if (!name) {
			name = 'untitled';
		}
		gCZIOUtils.AppendToFile(paletteFile, r + " " + g + " " + b + "\t" + name + "\n");	
	},

	AddColorToHistory : function(col, name) {
		if ((this.prevHistoryColor != null) && 
			(this.prevHistoryColor == col) && 
			(this.prevHistoryColorName == name)) {
			// do not add two consecutive colors which are the same
			return; 
		}

		this.AddColorToPalette(col, name, 'History.gpl', czGetLocaleString('history'));

		this.prevHistoryColor = col;
		this.prevHistoryColorName = name;
	},

	AddColorToFavorites : function(col, name) {
		this.AddColorToPalette(col, name, 
					'Favorites.gpl', czGetLocaleString('favorites'));
	},

	// takes the color info from the color picker widgets
	AddPickerColorToFavorites : function() {
		var colorNameElem = czColorPickerGetEntryboxElem('color-name');
		this.AddColorToFavorites(gCZCurrentColor, colorNameElem.value);
	},

	CustomPaletteChangeViewImpl : function(viewMode) {
		var listBox = document.getElementById('colorzilla-palette-color-list-container');
		var customPalette = document.getElementById('colorzilla-palette-custom-palette-panel');
	
		if (viewMode == 'list') {
			customPalette.setAttribute('hidden', "true");
			listBox.setAttribute('hidden', "false");
		} else {
			listBox.setAttribute('hidden', "true");
			customPalette.setAttribute('hidden', "false");
		}

		this.ClearPaletteMarkers();

		czWritePref("last-palette-view-mode", viewMode);
	},

	CustomPaletteChangeView : function (event) {
		var viewMode = event.target.value;
		this.CustomPaletteChangeViewImpl(viewMode);
	},

	CustomColorClicked : function (event) {
		var target = event.target;
		if (target.tagName == 'spacer') {
			// this click came from a thumbnail view
			// highlight the current tile
			target.setAttribute("current", "true");
			if (this.previouslyMarkedColorTile != null) {
				// clear the previously marked item
				this.previouslyMarkedColorTile.removeAttribute("current");
			}
			this.previouslyMarkedColorTile = target;
		}	

		var color = event.target.value;
		if (color) {
			var colorHex = color.value;
			var newColor = czRGBHexaAttributeToCol(colorHex);
			this.currentColorIndex = color.indexInPalette;
			czUpdateColorPicker(newColor, false, false, event.target.value.name);
		}
	},

	SwitchToPaletteImpl : function(palette) {
		// save the current palette metadata
		this.currentPaletteMetaData = palette;

		var filename = palette.getAttribute('filename');
		var file = Components.classes["@mozilla.org/file/local;1"].
				createInstance(Components.interfaces.nsILocalFile);
		file.initWithPath(filename);
		var colorPalette = new CZColorPalette();
		colorPalette.FromFile(file);	

		// save the current palette
		this.currentPalette = colorPalette;
		this.FillList(colorPalette);
		this.FillCustomPalette(colorPalette);

		czWritePref("last-palette", palette.getAttribute('name'));
	},

	SwitchToPalette : function(event) {
		var palette = event.target.value;
		this.SwitchToPaletteImpl(palette);
	},

	OpenPalettesFolder : function(event) {
		var folder = gCZIOUtils.GetPalettesFolder().QueryInterface(Components.interfaces.nsILocalFile);
		if (iosartGetPlatform() == 'windows') {
			// open the windows explorer with our folder
			folder.launch();
		} else {
			var path = folder.path;
			var pathMessage = czGetLocaleString('palettesFolderPathMessage', [path]);
			// copy the path to the clipboard
			czCopyStringToClipboard(path);
			czShowAlert(window, 
				czGetLocaleString('palettesFolder'),
				pathMessage);
			
		}
	},

	ReloadCurrentPalette : function() {
		// switch to the current palette (reloading it):
		if (this.currentPaletteMetaData) {
			this.SwitchToPaletteImpl(this.currentPaletteMetaData);
		}
	}, 

	DeleteColorRange : function(firstColor, numberOfColors) {
		if (this.currentPalette == null) {
			// strange, no palette selected
			dump("warning: no palette selected\n");
			return;
		}

		// remove the colors from the palette
		this.currentPalette.colors.splice(firstColor, numberOfColors);

		var paletteFilename = this.currentPaletteMetaData.getAttribute('filename');
		var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
		file.initWithPath(paletteFilename);

		// dump the palette to file
		this.currentPalette.ToFile(file);
	
		this.ReloadCurrentPalette();
	},

	DeleteOneColor : function() {
		var indexToDelete = this.currentColorIndex;
		if (indexToDelete == -1) {
			// no index selected
			return;
		}

		// delete one color at the current index
		this.DeleteColorRange(indexToDelete, 1);
	},

	DeleteAllColors : function() {
		if (this.currentPalette == null) {
			// strange, no palette selected
			dump("warning: no palette selected\n");
			return;
		}

		// delete all the colors from the current palette
		this.DeleteColorRange(0, this.currentPalette.colors.length);
	},

    ExportPalette : function() {
		if (this.currentPalette == null) {
			// strange, no palette selected
			dump("warning: no palette selected\n");
			return;
		}

        var nsIFilePicker = Components.interfaces.nsIFilePicker;
        var fp = Components.classes["@mozilla.org/filepicker;1"]
                            .createInstance(nsIFilePicker);
        fp.defaultExtension = "css";
        fp.appendFilter(czGetLocaleString('cssFiles'), '*.css')
        fp.init(window, czGetLocaleString('saveAs'), nsIFilePicker.modeSave);

        var rv = fp.show();
        if (rv == nsIFilePicker.returnCancel) {
                return;
        }

  		this.currentPalette.ToCSSFile(fp.file);	    
    },

	SavePaletteAs : function() {
		if (this.currentPalette == null) {
			// strange, no palette selected
			dump("warning: no palette selected\n");
			return;
		}

		var newName = czShowPrompt(window, 
						czGetLocaleString('paletteNameTitle'), 
						czGetLocaleString('paletteNamePrompt')); 
		if ((newName == null) || (newName.length == 0)) {
			return;
		}

		// change the palette name in place
		var oldName = this.currentPalette.name;
		this.currentPalette.name = newName;
	
		// normalize the answer to get a filename	
		var filename = newName.replace(/[^A-Za-z0-9_]/g, "_");	
		filename += ".gpl";
		var finalFile = gCZIOUtils.GetPalettesFolder();
		finalFile.append(filename);

		// dump the palette to file
		this.currentPalette.ToFile(finalFile);	

		// restore the name of this palette for good order
		this.currentPalette.name = oldName;

		var paletteSavedMessage = czGetLocaleString('palleteSavedMessage', [filename]);
		czShowAlert(window, 
				czGetLocaleString('paletteSaved'),
				paletteSavedMessage);
	}

}

var gCZCustomPaletteUI = new CZColorPaletteUI();
var gCZIOUtils = new CZIOUtils();


// var file = gCZIOUtils.GetPalettesFolder();
// file.append('palette.gpl');
// pal = new CZColorPalette();
// pal.ReSortByHue(file);

