/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is ColorZilla Mozilla Extension.
 * The Initial Developer of the Original Code is Alex Sirota <alex@elbrus.com>. 
 * Portions created by Alex Sirota are Copyright (C) 2004 Alex Sirota. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function iosartIsInFirefox() {
	return (navigator.userAgent.search(/Firefox/gi) != -1);
}

function iosartIsInThunderbird() {
	return (navigator.userAgent.search(/Thunderbird/gi) != -1);
}

function iosartIsInMozilla() {
	return (!iosartIsInFirefox() && !iosartIsInThunderbird());
}

function iosartGetPlatform() {
	var platform = navigator.platform.toLowerCase();
	
	if (platform.indexOf('linux') != -1) {
		return 'linux';
	}

	if (platform.indexOf('mac') != -1) {
		return 'mac';
	}

	return 'windows';
}

function iosartGetPlatformFull() {
	return navigator.platform.toLowerCase();
}


function iosartGetProfileDir() {
   var dirServiceProp = Components.classes["@mozilla.org/file/directory_service;1"];
   dirServiceProp = dirServiceProp.getService(Components.interfaces.nsIProperties);
   var profileDir = dirServiceProp.get("ProfD", Components.interfaces.nsIFile);
   return profileDir;
}

function iosartGetHome(extensionGUID) {
   var profileDir = iosartGetProfileDir();
   var home = profileDir.clone();
   home.append('extensions');
   home.append(extensionGUID);

   return home;
}

function iosartGetComponentFile(extensionGUID, filename) {
   var home = iosartGetHome(extensionGUID);
   var componentFile = home.clone();
   componentFile.append('components');
   componentFile.append(filename);
   return componentFile;
}

function iosartRemoveRegistryFiles() {
   var profileDir = iosartGetProfileDir();
   var compregFile = profileDir.clone();
   compregFile.append('compreg.dat');
   compregFile.remove(false);
   dump("\nRemoved " + compregFile.path + "\n");

   var xptiFile = profileDir.clone();
   xptiFile.append('xpti.dat');
   xptiFile.remove(false);
   dump("\nRemoved " + xptiFile.path + "\n");
}

function iosartInstallPlatformLibraryIfNeeded(packageName, extensionGUID) {
   if (iosartIsInMozilla()) {
	// we are in Mozilla, installed properly through install.js
	return false;
   }

   var platform = iosartGetPlatform();
   var platformFull = iosartGetPlatformFull();

   if (platform == 'windows') {
	// the default dll is for windows
	return false;
   }

   if (platformFull == 'macintel') {
        platform = platformFull;
   }

   var targetFileName = packageName + ".dll";

   var platformSpecificFileName = packageName + ".dll." + platform;
   var platformSpecificFile = iosartGetComponentFile(extensionGUID, platformSpecificFileName);
   if (!platformSpecificFile.exists()) {
	// the file doesn't exist, we already renamed it
 	return false;
   }

   // the file exists, rename it, so it becomes the main library file
   dump("\nRenaming " + platformSpecificFileName + " to " + targetFileName + "\n");

   platformSpecificFile.moveTo(null, targetFileName);

   // re-get the file (because it now points to the new name):
   platformSpecificFile = iosartGetComponentFile(extensionGUID, platformSpecificFileName);
   if (platformSpecificFile.exists()) {
	// the file still exists, something's wrong
	dump("\nFailed renaming " + platformSpecificFileName + " to " + targetFileName + "\n");
	return false; 
   }

   // load the dll so the user doesn't have to restart the browser to see the change
   var mainDLLFile = iosartGetComponentFile(extensionGUID, targetFileName);
   var componentRegistrar = Components.manager.QueryInterface(Components.interfaces.nsIComponentRegistrar);
   try {
       componentRegistrar.autoRegister(mainDLLFile);
   } catch(err) {
     	dump("\nFailed autoRegistering renamed platform specific " + targetFileName + "\n");
	return false; 
   }

   // the file existed, we have renamed (installed) it
   // remove compreg.dat and xpti.dat files, so on the next install
   // the file will get registered
   // remark: with auto-registration above, this probably is no longer needed
   iosartRemoveRegistryFiles();

   return true;
}

