/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is ColorZilla Mozilla Extension.
 * The Initial Developer of the Original Code is Alex Sirota <alex@elbrus.com>. 
 * Portions created by Alex Sirota are Copyright (C) 2004 Alex Sirota. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function czInitComponent() {
	try {
		const cid = "@iosart.com/Utils/ColorZilla;1";
		gCZComponent = Components.classes[cid].createInstance();
		gCZComponent = gCZComponent.QueryInterface(Components.interfaces.mozIColorZilla);
	} catch (err) {
		dump("Couldn't get object: " + err + "\n");
		return;
	}
}

function czGetScreenColor(x, y) {
	var col = gCZComponent.GetPixel(x, y);
	return col;
}




function czGetLocaleString(strName, formattingValues) {
	var str = null;
	try {
		var strbundle = document.getElementById("colorzilla-strings");
		var strbundleFallback = document.getElementById("colorzilla-strings-fallback");
		if (formattingValues === undefined) {
            try {
    			str = strbundle.getString(strName);
            } catch(e) {
    			str = strbundleFallback.getString(strName);
            }
		} else {
            try {
    			str = strbundle.getFormattedString(strName, formattingValues);
            } catch(e) {
    			str = strbundleFallback.getFormattedString(strName, formattingValues);
            }
		}
	} catch (err) {
		dump("Couldn't get string: " + strName + "\n");
	}
	return str;
}


function czCopyStringToClipboard(str) {
	var clipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"];
	clipboardHelper = clipboardHelper.getService(Components.interfaces.nsIClipboardHelper);

     	clipboardHelper.copyString(str);
}

function czHSVTriple(h, s, v) {
	this.h = h;
	this.s = s;
	this.v = v;

	this.toString = function () {
		return "(" + this.h + ", " + this.s + ", " + this.v + ")";
	}
}

function czRGBTriple(r, g, b) {
	this.r = r;
	this.g = g;
	this.b = b;

	this.toString = function () {
		return "(" + this.r + ", " + this.g + ", " + this.b + ")";
	}
}

// RGB -> COLORREF
function czRGBToColor(r, g, b) {
	return r | (g << 8) | (b << 16);
}

// COLORREF -> R/G/B:
function czGetRValue(color) {
	return color & 0xff;
}

function czGetGValue(color) {
	return (color >> 8) & 0xff;
}

function czGetBValue(color) {
	return (color >> 16) & 0xff;
}

// 0..255 -> 0% - 100%
function czIntToPercent(val) {
	return Math.floor(((val * 100) / 255) + 0.5);
}

// 0..255 -> 00 - ff 
function czDecimalToHexa(val) {
	var hexStr = val.toString(16);
	if (hexStr.length < 2) {
		hexStr = "0" + hexStr;
	}
	return hexStr;
}

// COLORREF -> rgb(100%, 100%, 100%)
function czColToRGBPercentageAttribute(col) {
 	var colStr = czIntToPercent(czGetRValue(col)) + "%, " + 
			czIntToPercent(czGetGValue(col)) + "%, " + 
			czIntToPercent(czGetBValue(col)) + "%";
	return "rgb(" + colStr + ")";
}

// COLORREF -> rgb(255, 255, 255)
function czColToRGBAttribute(col) {
 	var colStr = czGetRValue(col) + ", " + czGetGValue(col) + ", " + czGetBValue(col);
	return "rgb(" + colStr + ")";
}

// rgb(255, 255, 255) -> COLORREF
function czRGBAttributeToCol(colAttribute) {
	var firstParen = colAttribute.split("(");
	colAttribute = firstParen[1];
	var secondParen = colAttribute.split(")");
	colAttribute = secondParen[0];
	var rgbArr = colAttribute.split(",");
	return czRGBToColor(rgbArr[0], rgbArr[1], rgbArr[2]);
}


// COLORREF -> #ffffff
function czColToRGBHexaAttribute(col) {
 	var colStr = czDecimalToHexa(czGetRValue(col)) + 
			czDecimalToHexa(czGetGValue(col)) + 
			czDecimalToHexa(czGetBValue(col));
	if (!gbCZLowerCaseHexa) {
		colStr = colStr.toUpperCase();
	}
	return "#" + colStr;
}

// #ffffff -> COLORREF
// assumev valid string
function czRGBHexaAttributeToCol(col) {
	var red   = col.substr(1, 2);
	var green = col.substr(3, 2);
	var blue  = col.substr(5, 2);

	red   = parseInt(red, 16);
	green = parseInt(green, 16);
	blue  = parseInt(blue, 16);

	return czRGBToColor(red, green, blue);
}

function czRGBToGrayscale(r, g, b) {
	return (r * 0.30) + (g * 0.59) + (b * 0.11);
}

function czRGBToHSV(r, g, b) {
	var hue, sat, val;
	var max, dif;

	max = Math.max(r, g, b);
	dif = max - Math.min(r, g, b);
	sat = (max == 0) ? 0 : (255 * dif/max);

	if (sat == 0) {
		hue=0;
	} else if (r == max) {
		hue = 60.0 * (g - b) / dif;
	} else if (g == max) {
		hue = 120.0 + 60.0*(b - r) / dif;
	} else if (b == max) {
		hue = 240.0 + 60.0*(r - g) / dif;
	}

	if (hue < 0.0) {
		hue += 360.0;
	}
	hue = Math.round((hue * 255.0) / 360.0);
	sat = Math.round(sat);

	val = max;
	return new czHSVTriple(hue, sat, val)
}


function czHSVToRGB(hue, sat, val) {
	var r, g, b;
	var i, f, p, q, t;

	if (sat == 0) {
		r = g = b = val;
	} else {
		hue = ((hue * 359) / (255*60));
		sat /= 255;
		val /= 255;

		i = Math.floor(hue);
		f = hue - i;
		p = val * (1 - sat);
		q = val * (1 - (sat * f));
		t = val * (1 - sat * (1 - f));

		switch (i) {
			case 0 : r = val; g = t;   b = p;   break;
			case 1 : r = q;   g = val; b = p;   break;
			case 2 : r = p;   g = val; b = t;   break;
			case 3 : r = p;   g = q;   b = val; break;
			case 4 : r = t;   g = p;   b = val; break;
			default: r = val; g = p;   b = q;
		}
		r = Math.round(r * 255);
		g = Math.round(g * 255);
		b = Math.round(b * 255);
	}
	return new czRGBTriple(r, g, b);
}


// this is actually a "value" component of HSV:
function czGetColorLightness(color) {
	var r = czGetRValue(color);
	var g = czGetGValue(color);
	var b = czGetBValue(color);
	return Math.max(r, g, b);
}

// #ff0000 -> red
function czHexaAttributeToPredefinedColor(hexCol) {
	hexCol = hexCol.toLowerCase();
  	if (hexCol == "#800000") { return "maroon"; }
  	if (hexCol == "#ff0000") { return "red"; }
  	if (hexCol == "#ffa500") { return "orange"; }
  	if (hexCol == "#ffff00") { return "yellow"; }
  	if (hexCol == "#808000") { return "olive"; }
  	if (hexCol == "#800080") { return "purple"; }
  	if (hexCol == "#ff00ff") { return "fuchsia"; }
  	if (hexCol == "#ffffff") { return "white"; }
  	if (hexCol == "#00ff00") { return "lime"; }
  	if (hexCol == "#008000") { return "green"; }
  	if (hexCol == "#000080") { return "navy"; }
  	if (hexCol == "#0000ff") { return "blue"; }
  	if (hexCol == "#00ffff") { return "aqua"; }
  	if (hexCol == "#008080") { return "teal"; }
  	if (hexCol == "#000000") { return "black"; }
  	if (hexCol == "#c0c0c0") { return "silver"; }
  	if (hexCol == "#808080") { return "gray"; }
	return null;
}

function czFixHexValue(val) {
	if ((val.length != 7) || (val.substr(0, 1) != "#")) {
		return "#000000";
	}
	return val;
}

function czFixByteValue(val) {
	if (val > 255) {
		val = 255;
	} else {
		if (val < 0) {
			val = 0;
		}
	}

	return val;
}

function czValidateByteValue(val) {
	if ((val < 0) || (val > 255)) {
		return false;
	}
	return true;
}

function czWritePref(key, value) {
   try {
     gCZPref.setCharPref(key, value);
   } 
   catch (err) {
     dump("\nSetting preference for key " + key + " (" + value + ") failed: " + err + "\n");
   }
   try {
	gCZPrefService.savePrefFile(null);
   }
   catch (err) {
        dump("\nSaving preferences failed: " + err + "\n");
   }
}

function czReadPref(key, defaultVal) {
   var val;
   try {
     val = gCZPref.getCharPref(key);
   } 
   catch (err) {
     val = defaultVal; 
   }
   return val;
}

function czClearPref(key) {
   try {
	gCZPref.clearUserPref(key);
   } 
   catch (err) {
	dump("\nCouldn't clear user pref: " + key + "\n");		
   }
   try {
	gCZPrefService.savePrefFile(null);
   }
   catch (err) {
        dump("\nSaving preferences failed: " + err + "\n");
   }
}


function czWindowsShouldOpenInNewTab() {
	return czReadPref("open_window_in_tab", "false") == "true";
}

function czGoToURL(url) {
	if (czWindowsShouldOpenInNewTab()) {
		var newTab = window.getBrowser().addTab(url);
	        window.getBrowser().selectedTab = newTab;
	} else {
		openDialog("chrome://browser/content/browser.xul", "_blank", "chrome,all,dialog=no", url, null, null);
		//window.open(url);
	}
}

function czGoToURLAndClose(url) {
	if (czWindowsShouldOpenInNewTab()) {
		if (window.opener.getBrowser) {
			// we were opened by a browser
	        	var newTab = window.opener.getBrowser().addTab(url);
       			window.opener.getBrowser().selectedTab = newTab;
		} else {
			// we were opened by the extensions window
        		window.opener.open(url);
		}
        } else {
        	window.opener.open(url);
        }

    	window.close();
}

function czOpenAboutDialog() {
  window.open('chrome://colorzilla/content/about.xul','','chrome,centerscreen');
}

function czOpenConfigShortcutsDialog(callBack) {
 window.openDialog('chrome://colorzilla/content/colorzillaShortcutDialog.xul',
							'',
							'chrome,centerscreen',
							callBack);
}

function czShowAlert(parent, title, text) {
	var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"];
	promptService = promptService.getService(Components.interfaces.nsIPromptService);
	promptService.alert(parent, title, text);
}

function czShowPrompt(parent, title, text, initialValue) {
	var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"];
	promptService = promptService.getService(Components.interfaces.nsIPromptService);

	if (!initialValue) {
		initialValue = "";
	}
	var answer = { value: initialValue };
    	var dummy = { value: false };
	var bOK = promptService.prompt(window, title, text, answer, null, dummy);

	if (bOK) {
		return answer.value;
	} else {
		return null;
	}
}

function czCompareTwoStrings(a, b) {
	if ( a < b ) return -1;
        if ( a > b ) return 1;
        return 0;
}

function szClipString(text, length) {
    if (typeof length == 'undefined') {
        length = 15;
    }
    var clippedText; 
    if (text.length > length) {
        clippedText = text.substr(0,length) + "...";
    } else {
        clippedText = text;
    }
    return clippedText;
}


var gCZPrefService = Components.classes["@mozilla.org/preferences-service;1"]
                            .getService(Components.interfaces.nsIPrefService);
var gCZPref = gCZPrefService.getBranch("colorzilla."); 

// the following should be loaded from preferences on init
var gbCZLowerCaseHexa = (czReadPref("lowercase-hexa", "false") == 'true');
